@@ -9,7 +9,7 @@
 
 /* Add a new node */
 - addNode:(STR)aCharPointer
-  { return [self filter:[Node str:aCharPointer]]; }
+  { return [self filter:[Graphnode str:aCharPointer]]; }
 
 /* Add a new node and define it */
 - defineNode:(STR)aCharPointer
