@@ -3,8 +3,8 @@
 
 #include "objpak.h"
 
-@interface Node : String
-  { BOOL isDefined, isMarked; id references; }
+@interface Graphnode : String
+  { BOOL isDefined; BOOL isMarked; id references; }
   
 - addReference:anotherNode;
 - mark;
