@@ -1,7 +1,23 @@
 
 /*
- * Computer Algebra Kit (c) 1993,99 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: ccomplex.h,v 1.2 1999/07/06 19:48:08 stes Exp $
+ * Computer Algebra Kit (c) 1993,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: ccomplex.h,v 1.7 2001/07/09 19:30:49 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #ifndef __CACOMPLEX_HEADER__
@@ -9,7 +25,7 @@
 
 #include "cobject.h"
 
-@interface Complex : CAObject
+@interface Gaussian : CAObject
 {
   id real;
   id imaginary;
@@ -33,7 +49,6 @@
 - (BOOL) isEqual:b;
 
 - conjugate;
-- conjugateSelf;
 - norm;
 - asReal;
 
@@ -52,16 +67,13 @@
 - (BOOL) isUnit;
 - square;
 - multiplyReal:b;
-- multiplySelfReal:b;
 - multiply:b;
 - power:(int)n;
 
 - divideReal:b;
-- divideSelfReal:b;
 - divide:b;
 - inverse;
 - quotientReal:b;
-- quotientSelfReal:b;
 - quotient:b;
 - remainder:b quotient:(id *)q;
 
