@@ -1,7 +1,23 @@
 
 /*
- * Computer Algebra Kit (c) 1992,99 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: integer.m,v 1.2 1999/07/06 19:48:09 stes Exp $
+ * Computer Algebra Kit (c) 1992,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: integer.m,v 1.7 2000/10/12 14:40:25 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "cakit.h"
@@ -1808,7 +1824,7 @@ static void
 bigint_fileout (id aFiler, bigint_c * a, bigint_args args)
 {
   DIGIT *digits;
-  int sign, count;
+  int sign, count, i;
   assert (bigint_check (a, args));
 
   sign = a->sign;
@@ -1818,8 +1834,8 @@ bigint_fileout (id aFiler, bigint_c * a, bigint_args args)
 
   [aFiler fileOut:&sign type:'i'];
   [aFiler fileOut:&count type:'i'];
-  while (count--)
-    [aFiler fileOut:digits++ type:'S'];
+  for (i = 0; i < count; i++)
+    [aFiler fileOut:digits + i type:'S'];
   OC_Free (digits);
   assert (bigint_check (a, args));
 }
@@ -1895,11 +1911,11 @@ bigint_filein (id aFiler, bigint_c * c, bigint_args args)
 
       for (i = 2; j = i * (i + 1), j < USHRT_MAX && i + 1 <= m; i += 2)
 	{
-	  fac = [fac multiplySelfDigit:j];
+	  fac = [fac multiplyDigit:j];
 	}
 
       for (; i <= m; i++)
-	fac = [fac multiplySelfDigit:i];
+	fac = [fac multiplyDigit:i];
 
       return fac;
     }
@@ -1929,37 +1945,49 @@ bigint_filein (id aFiler, bigint_c * c, bigint_args args)
   return [super release];
 }
 
-- empty
+- doempty
 {
-  self = [self clone];
   bigint_oozero (&value, bigint_getargs (self));
   return self;
 }
 
+- empty
+{
+  return [[self clone] doempty];
+}
+
 - (bigint_t) bigint_value
 {
   return &value;
 }
 
-- bigint_value:(bigint_t)aValue
+- dobigint_value:(bigint_t)aValue
 {
-  self = [self clone];
   bigint_copy (&value, aValue, bigint_getargs (self));
   return self;
 }
 
+- bigint_value:(bigint_t)aValue
+{
+  return [[self clone] dobigint_value:aValue];
+}
+
 - (bigint_t) bigint_reference
 {
   return &value;
 }
 
-- bigint_reference:(bigint_t)aReference
+- dobigint_reference:(bigint_t)aReference
 {
-  self = [self clone];
   bigint_move (&value, aReference, bigint_getargs (self));
   return self;
 }
 
+- bigint_reference:(bigint_t)aReference
+{
+  return [[self clone] dobigint_reference:aReference];
+}
+
 - (BOOL)sameClass:b			
 {
   return isa == [b class];
@@ -2128,12 +2156,12 @@ getChars (id self, int *cref, DIGIT base)
   count = 0;
   stack = (char *) OC_MallocAtomic (sizeof (char) * (capacity = 100));
 
-  copy = [copy quotientSelfDigit:base remainder:&r];
+  copy = [copy quotientDigit:base remainder:&r];
   stack [count++] = r + '0';
 
   while ([copy notZero])
     {
-      copy = [copy quotientSelfDigit:base remainder:&r];
+      copy = [copy quotientDigit:base remainder:&r];
       stack [count++] = r + '0';
       if (count == capacity)
 	stack = (char *) OC_Realloc (stack, capacity *= 2);
@@ -2159,7 +2187,7 @@ fromDigits (id self, STR aString)
     {
       if (isdigit (c))
 	{
-	  res = [[res multiplySelfDigit:10] addSelfDigit:c - '0'];
+	  res = [[res multiplyDigit:10] addDigit:c - '0'];
 	}
       else
 	{
@@ -2197,7 +2225,7 @@ fromDigits (id self, STR aString)
 	  if (isdigit (c))
 	    {
 	      id res = fromDigits (self, aString);
-	      return (sign == +1) ? res : [res negateSelf];
+	      return (sign == +1) ? res : [res negate];
 	    }
 	  else
 	    {
@@ -2233,9 +2261,7 @@ fromDigits (id self, STR aString)
 }
 - digitValue:(DIGIT)aValue
 {
-  self = [self empty];
-  [self insertDigit:aValue];
-  return self;
+  return [[self empty] insertDigit:aValue];
 }
 - (unsigned long) ulongValue
 {
@@ -2290,7 +2316,7 @@ fromDigits (id self, STR aString)
 
   if (aValue < 0)
     {
-      res = [[self ulongValue:-aValue] negateSelf];
+      res = [[self ulongValue:-aValue] negate];
     }
   else
     {
@@ -2433,20 +2459,6 @@ fromDigits (id self, STR aString)
     }
 }
 
-- negateSelf
-{
-  if ([self isZero] || [self isCharacteristicTwo])
-    {
-      return self;
-    }
-  else
-    {
-      assert ([self check]);
-      bigint_inneg (bigint_id2c (self), bigint_getargs (self));
-      return [self invalidate];
-    }
-}
-
 - _double:(int)v
 {
   if ([self isZero] || [self isCharacteristicTwo])
@@ -2462,20 +2474,6 @@ fromDigits (id self, STR aString)
     }
 }
 
-- _doubleSelf:(int)v
-{
-  if ([self isZero] || [self isCharacteristicTwo])
-    {
-      return [self zero];
-    }
-  else
-    {
-      assert ([self check]);
-      bigint_indbl (bigint_id2c (self), v, bigint_getargs (self));
-      return [self invalidate];
-    }
-}
-
 - double
 {
   if ([self isZero] || [self isCharacteristicTwo])
@@ -2491,20 +2489,6 @@ fromDigits (id self, STR aString)
     }
 }
 
-- doubleSelf
-{
-  if ([self isZero] || [self isCharacteristicTwo])
-    {
-      return [self zero];
-    }
-  else
-    {
-      assert ([self check]);
-      bigint_indbl (bigint_id2c (self), +1, bigint_getargs (self));
-      return [self invalidate];
-    }
-}
-
 - _add:(int)v:b:(int)w
 {
   assert ([self sameClass:b] && v * v == 1 && w * w == 1);
@@ -2533,30 +2517,6 @@ fromDigits (id self, STR aString)
   return nil;			/* to quiet some compilers, e.g. WATCOM */
 }
 
-- _addSelf:(int)v:b:(int)w
-{
-  assert ([self sameClass:b] && v * v == 1 && w * w == 1);
-
-  if (self == b)
-    {
-      return (v == w) ? [self _doubleSelf : v]:[self zero];
-    }
-  else
-    {
-      if ([self inAdditiveMonoid])
-	{
-	  if ([b isZero])
-	    return [self _negateSelf:v];
-	  if ([self isZero])
-	    return [b _negate:w];
-	}
-      bigint_inadd (bigint_id2c (self), v, bigint_id2t (b), w, bigint_getargs (self));
-      return [self invalidate];
-    }
-
-  return nil;
-}
-
 - add:b
 {
   [self checkSameClass:b];
@@ -2585,30 +2545,6 @@ fromDigits (id self, STR aString)
   return nil;
 }
 
-- addSelf:b
-{
-  assert ([self sameClass:b]);
-
-  if (self == b)
-    {
-      return [self doubleSelf];
-    }
-  else
-    {
-      if ([self inAdditiveMonoid])
-	{
-	  if ([b isZero])
-	    return self;
-	  if ([self isZero])
-	    return b;
-	}
-      bigint_inadd (bigint_id2c (self), +1, bigint_id2t (b), +1, bigint_getargs (self));
-      return [self invalidate];
-    }
-
-  return nil;
-}
-
 - subtract:b
 {
   [self checkSameClass:b];
@@ -2637,30 +2573,6 @@ fromDigits (id self, STR aString)
   return nil;
 }
 
-- subtractSelf:b
-{
-  assert ([self sameClass:b]);
-
-  if (self == b)
-    {
-      return [self zero];
-    }
-  else
-    {
-      if ([self inAdditiveMonoid])
-	{
-	  if ([b isZero])
-	    return self;
-	  if ([self isZero])
-	    return [b negate];
-	}
-      bigint_inadd (bigint_id2c (self), +1, bigint_id2t (b), -1, bigint_getargs (self));
-      return [self invalidate];
-    }
-
-  return nil;
-}
-
 - addDigit:(DIGIT)d
 {
   bigint_c c;
@@ -2672,37 +2584,17 @@ fromDigits (id self, STR aString)
   return [self bigint_reference:&c];
 }
 
-- addSelfDigit:(DIGIT)d
-{
-  if (d == 0)
-    return self;
-  if ([self isZero])
-    return [self digitValue:d];
-  bigint_inaddword (&value, +1, d, +1, bigint_getargs (self));
-  return [self invalidate];
-}
-
 - subtractDigit:(DIGIT)d
 {
   bigint_c c;
   if (d == 0)
     return self;
   if ([self isZero])
-    return [[self digitValue:d] negateSelf];
+    return [[self digitValue:d] negate];
   bigint_ooaddword (&c, &value, +1, d, -1, bigint_getargs (self));
   return [self bigint_reference:&c];
 }
 
-- subtractSelfDigit:(DIGIT)d
-{
-  if (d == 0)
-    return self;
-  if ([self isZero])
-    return [[self digitValue:d] negateSelf];
-  bigint_inaddword (&value, +1, d, -1, bigint_getargs (self));
-  return [self invalidate];
-}
-
 
 - one
 {
@@ -2754,30 +2646,6 @@ fromDigits (id self, STR aString)
   return nil;
 }
 
-- _addSelf:(int)v:B multiply:b:(int)w
-{
-  assert ([self sameClass:B] && [self sameClass:b] && v * v == 1 && w * w == 1);
-
-  if ([self isZero])
-    return [B _multiply:b:+w];
-  if ([B isZero])
-    return [self _negateSelf:v];
-  if ([b isZero])
-    return [self _negateSelf:v];
-
-  if ([b isOne])
-    return [self _addSelf:v:B:+w];
-  if ([B isOne])
-    return [self _addSelf:v:b:+w];
-  if ([b isMinusOne])
-    return [self _addSelf:v:B:-w];
-  if ([B isMinusOne])
-    return [self _addSelf:v:b:-w];
-
-  bigint_inaddmul (bigint_id2c (self), v, bigint_id2t (B), bigint_id2t (b), w, bigint_getargs (self));
-  return [self invalidate];
-}
-
 - _multiply:a:(int)v add:B:(int)w
 {
   assert ([self sameClass:a] && [self sameClass:B] && v * v == 1 && w * w == 1);
@@ -2808,30 +2676,6 @@ fromDigits (id self, STR aString)
   return nil;
 }
 
-- _multiplySelf:a:(int)v add:B:(int)w
-{
-  assert ([self sameClass:a] && [self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([self isZero])
-    return [B _negate:+w];
-  if ([a isZero])
-    return [B _negate:+w];
-  if ([B isZero])
-    return [self _multiplySelf:a:+v];
-
-  if ([self isOne])
-    return [a _add:+v:B:+w];
-  if ([a isOne])
-    return [self _addSelf:+v:B:+w];
-  if ([self isMinusOne])
-    return [a _add:-v:B:+w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B:+w];
-
-  bigint_inmuladd (bigint_id2c (self), bigint_id2t (a), v, bigint_id2t (B), w, bigint_getargs (self));
-  return [self invalidate];
-}
-
 - _multiply:a:(int)v add:B multiply:b:(int)w
 {
   assert ([self sameClass:a] && [self sameClass:B] && [self sameClass:b] && v * v == 1 && w * w == 1);
@@ -2873,42 +2717,6 @@ fromDigits (id self, STR aString)
   return nil;
 }
 
-- _multiplySelf:a:(int)v add:B multiply:b:(int)w
-{
-  assert ([self sameClass:a] && [self sameClass:B] && [self sameClass:b] && v * v == 1 && w * w == 1);
-
-  if ([self isZero])
-    return [B _multiply:b:+w];
-  if ([a isZero])
-    return [B _multiply:b:+w];
-  if ([B isZero])
-    return [self _multiplySelf:a:+v];
-  if ([b isZero])
-    return [self _multiplySelf:a:+v];
-
-  if ([self isOne])
-    return [a _add:+v:B multiply:b:+w];
-  if ([a isOne])
-    return [self _addSelf:+v:B multiply:b:+w];
-  if ([b isOne])
-    return [self _multiplySelf:a:+v add:B:+w];
-  if ([B isOne])
-    return [self _multiplySelf:a:+v add:b:+w];
-
-  if ([self isMinusOne])
-    return [a _add:-v:B multiply:b:+w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B multiply:b:+w];
-  if ([b isMinusOne])
-    return [self _multiplySelf:a:+v add:B:-w];
-  if ([B isMinusOne])
-    return [self _multiplySelf:a:+v add:b:-w];
-
-  bigint_inmuladdmul (bigint_id2c (self), bigint_id2t (a), v, bigint_id2t (B), bigint_id2t (b), w, bigint_getargs (self));
-  return [self invalidate];
-}
-
-
 - (BOOL) isOne
 {
   assert ([self check]);
@@ -2935,28 +2743,8 @@ fromDigits (id self, STR aString)
 
 - square
 {
-  if ([self inMonoid] && [self isOne])
-    return self;
-  if ([self inAdditiveMonoid])
-    {
-      if ([self isMinusOne])
-	return [self one];
-      if ([self isZero])
-	return self;
-    }
-  if (1)
-    {
-      bigint_c c;
-      assert ([self check]);
-      bigint_oosqr (&c, bigint_id2t (self), bigint_getargs (self));
-      return bigint_c2id (self, &c);
-    }
+  bigint_c c;
 
-  return nil;
-}
-
-- squareSelf
-{
   if ([self inMonoid] && [self isOne])
     return self;
   if ([self inAdditiveMonoid])
@@ -2968,8 +2756,8 @@ fromDigits (id self, STR aString)
     }
 
   assert ([self check]);
-  bigint_insqr (bigint_id2c (self), bigint_getargs (self));
-  return [self invalidate];
+  bigint_oosqr (&c, bigint_id2t (self), bigint_getargs (self));
+  return bigint_c2id (self, &c);
 }
 
 - multiply:b
@@ -2982,6 +2770,7 @@ fromDigits (id self, STR aString)
     }
   else
     {
+      bigint_c c;
       if ([self inMonoid])
 	{
 	  if ([self isOne])
@@ -2998,53 +2787,11 @@ fromDigits (id self, STR aString)
 	  if ([b isMinusOne])
 	    return [self negate];
 	}
-      if (1)
-	{
-	  bigint_c c;
-	  bigint_oomul (&c, bigint_id2t (self), bigint_id2t (b), bigint_getargs (self));
-	  return bigint_c2id (self, &c);
-	}
+      bigint_oomul (&c, bigint_id2t (self), bigint_id2t (b), bigint_getargs (self));
+      return bigint_c2id (self, &c);
     }
-
-  return nil;
 }
 
-- multiplySelf:b
-{
-  assert ([self sameClass:b]);
-
-  if (self == b)
-    {
-      return [self squareSelf];
-    }
-  else
-    {
-      if ([self inMonoid])
-	{
-	  if ([b isOne])
-	    return self;
-	  if ([self isOne])
-	    return b;
-	}
-      if ([self inAdditiveMonoid])
-	{
-	  if ([self isZero])
-	    return self;
-	  if ([b isZero])
-	    return b;
-	  if ([b isMinusOne])
-	    return [self negateSelf];
-	  if ([self isMinusOne])
-	    return [b negate];
-	}
-      bigint_inmul (bigint_id2c (self), bigint_id2t (b), bigint_getargs (self));
-      return [self invalidate];
-    }
-
-  return nil;
-}
-
-
 - multiplyDigit:(DIGIT)d
 {
   bigint_c c;
@@ -3061,32 +2808,12 @@ fromDigits (id self, STR aString)
   if ([self isOne])
     return [self digitValue:d];
   if ([self isMinusOne])
-    return [[self digitValue:d] negateSelf];
+    return [[self digitValue:d] negate];
 
   bigint_oomulword (&c, &value, d, bigint_getargs (self));
   return [self bigint_reference:&c];
 }
 
-- multiplySelfDigit:(DIGIT)d
-{
-  if (d == 0)
-    return [self zero];
-  if (d == 1)
-    return [self negateSelf];
-  if (d == 2)
-    return [self doubleSelf];
-
-  if ([self isZero])
-    return [self zero];
-  if ([self isOne])
-    return [self digitValue:d];
-  if ([self isMinusOne])
-    return [[self digitValue:d] negateSelf];
-
-  bigint_inmulword (&value, d, bigint_getargs (self));
-  return [self invalidate];
-}
-
 - inverse
 {
   return ([self isOne] || [self isMinusOne]) ? self : nil;
@@ -3107,7 +2834,7 @@ fromDigits (id self, STR aString)
   if ([b isDigit])
     {
       id q = [self divideDigit:[b digitValue]];
-      return ([b sign] > 0) ? q : [q negateSelf];
+      return ([b sign] > 0) ? q : [q negate];
     }
   else
     {
@@ -3123,50 +2850,6 @@ fromDigits (id self, STR aString)
 	}
     }
 }
-- divideSelf:b
-{
-  if ([b isZero])
-    return nil;
-  if (self == b)
-    return [self one];
-  if ([b isOne])
-    return self;
-  if ([b isMinusOne])
-    return [self negateSelf];
-
-  if ([b isDigit])
-    {
-      self = [self divideSelfDigit:[b digitValue]];
-      return ([b sign] > 0) ? self : [self negateSelf];
-    }
-  else
-    {
-      return [self divide:b];
-    }
-}
-
-- quotientSelf:b
-{
-  if ([b isZero])
-    return nil;
-  if (self == b)
-    return [self one];
-  if ([b isOne])
-    return self;
-  if ([b isMinusOne])
-    return [self negateSelf];
-
-  if ([b isDigit])
-    {
-      self = [self quotientSelfDigit:[b digitValue]];
-      return ([b sign] > 0) ? self : [self negateSelf];
-    }
-  else
-    {
-      return [self quotient:b];
-    }
-}
-
 - divideDigit:(DIGIT)b
 {
   DIGIT r;
@@ -3188,35 +2871,10 @@ fromDigits (id self, STR aString)
     }
 }
 
-- divideSelfDigit:(DIGIT)b
-{
-  DIGIT r;
-
-  if (b == 0)
-    return nil;
-  if (b == +1)
-    return self;
-
-  self = [self quotientSelfDigit:b remainder:&r];
-  if (r)
-    {
-      return nil;
-    }
-  else
-    {
-      return self;
-    }
-}
-
 - quotientDigit:(DIGIT)b
 {
   return [self quotientDigit:b remainder:NULL];
 }
-- quotientSelfDigit:(DIGIT)d
-{
-  return [self quotientSelfDigit:d remainder:NULL];
-}
-
 - remainder:b quotient:(id *)q
 {
   if ([b isZero])
@@ -3256,44 +2914,6 @@ fromDigits (id self, STR aString)
     }
 }
 
-- remainderSelf:b quotient:(id *)q
-{
-  if ([b isZero])
-    return nil;
-  if ([b isOne])
-    {
-      if (q)
-	*q = self;
-      return [self zero];
-    }
-  if ([b isMinusOne])
-    {
-      if (q)
-	*q = [self negate];
-      return [self zero];
-    }
-
-  if ([self isZero])
-    {
-      if (q)
-	*q = [self zero];
-      return self;
-    }
-
-  if (q)
-    {
-      bigint_c d;
-      bigint_inremquo (&value, [b bigint_value], &d, bigint_getargs (self));
-      *q = [self bigint_reference:&d];
-      return [self invalidate];
-    }
-  else
-    {
-      bigint_inrem (&value, [b bigint_value], bigint_getargs (self));
-      return [self invalidate];
-    }
-}
-
 - quotientDigit:(DIGIT)d remainder:(DIGIT *)r
 {
   bigint_c q;
@@ -3328,38 +2948,6 @@ fromDigits (id self, STR aString)
   return [self bigint_reference:&q];
 }
 
-- quotientSelfDigit:(DIGIT)d remainder:(DIGIT *)r
-{
-  if (d == 0)
-    return nil;
-  if (d == 1)
-    {
-      if (r)
-	*r = 0;
-      return self;
-    }
-  if ([self isZero])
-    {
-      if (r)
-	*r = 0;
-      return self;
-    }
-
-  if (r)
-    {
-      bigintwrd_t rw;
-      bigint_inquoremword (&value, d, &rw, bigint_getargs (self));
-      assert (rw <= d);
-      *r = rw;
-    }
-  else
-    {
-      bigint_inquoremword (&value, d, NULL, bigint_getargs (self));
-    }
-
-  return [self invalidate];
-}
-
 - (DIGIT) remainderDigit:(DIGIT)d
 {
   if (d == 0)
@@ -3404,15 +2992,15 @@ fromDigits (id self, STR aString)
     }
   else
     {
-      self = [self copy];
+      id a = [self copy];
       b = [b copy];
       while ([b notZero])
 	{
-	  id t = [self remainderSelf:b];
-	  self = b;
+	  id t = [a remainder:b];
+	  a = b;
 	  b = t;
 	}
-      return self;
+      return a;
     }
 }
 
