@@ -104,7 +104,7 @@ bigintobj_ooobj (bigintobj_c * c, id obj, bigintobj_args args)
  * used like :
  *
  *              bigintobj_makeobj(c,args);
- *              bigintobj_inobj(c,[c->obj negateSelf],args)
+ *              bigintobj_inobj(c,[c->obj negate],args)
  */
 
 static void 
@@ -417,7 +417,7 @@ bigintobj_inneg (bigintobj_c * c, bigintobj_args args)
     }
   else
     {
-      bigintobj_inobj (c, [c->obj negateSelf], args);
+      bigintobj_inobj (c, [c->obj negate], args);
     }
 }
 
@@ -447,7 +447,7 @@ bigintobj_indbl (bigintobj_c * c, int v, bigintobj_args args)
     }
   else
     {
-      bigintobj_inobj (c, [c->obj _doubleSelf:v], args);
+      bigintobj_inobj (c, [c->obj _double:v], args);
     }
 }
 
@@ -481,7 +481,7 @@ bigintobj_inadd (bigintobj_c * c, int v, bigintobj_t b, int w, bigintobj_args ar
     {
       id B = bigintobj_obj (b, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj _addSelf:v:B:w], args);
+      bigintobj_inobj (c, [c->obj _add:v:B:w], args);
     }
 }
 
@@ -521,7 +521,7 @@ bigintobj_insqr (bigintobj_c * c, bigintobj_args args)
     }
   else
     {
-      bigintobj_inobj (c, [c->obj squareSelf], args);
+      bigintobj_inobj (c, [c->obj square], args);
     }
 }
 
@@ -555,7 +555,7 @@ bigintobj_inmul (bigintobj_c * c, bigintobj_t b, bigintobj_args args)
     {
       id B = bigintobj_obj (b, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj multiplySelf:B], args);
+      bigintobj_inobj (c, [c->obj multiply:B], args);
     }
 }
 
@@ -589,7 +589,7 @@ bigintobj_inmulv (bigintobj_c * c, bigintobj_t b, int v, bigintobj_args args)
     {
       id B = bigintobj_obj (b, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj _multiplySelf:B:v], args);
+      bigintobj_inobj (c, [c->obj _multiply:B:v], args);
     }
 }
 
@@ -625,7 +625,7 @@ bigintobj_inaddmul (bigintobj_c * c, int v, bigintobj_t b, bigintobj_t t, int w,
       id B = bigintobj_obj (b, args);
       id T = bigintobj_obj (t, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj _addSelf:v:B multiply:T:w], args);
+      bigintobj_inobj (c, [c->obj _add:v:B multiply:T:w], args);
     }
 }
 
@@ -661,7 +661,7 @@ bigintobj_inmuladd (bigintobj_c * c, bigintobj_t s, int v, bigintobj_t b, int w,
       id B = bigintobj_obj (b, args);
       id S = bigintobj_obj (s, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj _multiplySelf:S:v add:B:w], args);
+      bigintobj_inobj (c, [c->obj _multiply:S:v add:B:w], args);
     }
 }
 
@@ -699,7 +699,7 @@ bigintobj_inmuladdmul (bigintobj_c * c, bigintobj_t s, int v, bigintobj_t b, big
       id S = bigintobj_obj (s, args);
       id T = bigintobj_obj (t, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj _multiplySelf:S:v add:B multiply:T:w], args);
+      bigintobj_inobj (c, [c->obj _multiply:S:v add:B multiply:T:w], args);
     }
 }
 
@@ -733,7 +733,7 @@ bigintobj_inaddsqr (bigintobj_c * c, int v, bigintobj_t b, int w, bigintobj_args
     {
       id B = bigintobj_obj (b, args);
       bigintobj_makeobj (c, args);
-      bigintobj_inobj (c, [c->obj _addSelfSquare:v:B:w], args);
+      bigintobj_inobj (c, [c->obj _addSquare:v:B:w], args);
     }
 }
 
@@ -788,7 +788,7 @@ bigintobj_indiv (bigintobj_c * c, bigintobj_t b, bigintobj_args args)
     {
       id C, B = bigintobj_obj (b, args);
       bigintobj_makeobj (c, args);
-      C = [c->obj divideSelf:B];
+      C = [c->obj divide:B];
       if (C == nil)
 	{
 	  fprintf (stderr, "-exact division failed\n");
@@ -839,8 +839,24 @@ bigintobj_fileIn (id aFiler, bigintobj_c * c, bigintobj_args args)
 
 
 /*
- * Computer Algebra Kit (c) 1993,99 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: ivectorc.m,v 1.2 1999/07/06 19:48:10 stes Exp $
+ * Computer Algebra Kit (c) 1993,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: ivectorc.m,v 1.6 2000/10/12 14:40:26 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #if bigintobj_pervalue
@@ -881,13 +897,27 @@ bigintobjseq_check (bigintobjseq_t self, int n, bigintobj_args args)
 static bigintobjseq_t 
 bigintobjseq_alloc (int n, bigintobj_args args)
 {
-  return (bigintobjseq_t) OC_Malloc (n * bigintobj_bsize (args));
+  bigintobjseq_t p;
+  int m = n * bigintobj_bsize (args);
+#if OBJC_REFCNT
+  p = (bigintobjseq_t) OC_Calloc (m);
+#else
+  p = (bigintobjseq_t) OC_Malloc (m);
+#endif
+  return p;
 }
 
 static bigintobjseq_t 
-bigintobjseq_realloc (bigintobjseq_t self, int n, bigintobj_args args)
+bigintobjseq_realloc (bigintobjseq_t self, int oldc, int n, bigintobj_args args)
 {
-  return (bigintobjseq_t) OC_Realloc (self, n * bigintobj_bsize (args));
+  bigintobjseq_t p;
+  int m = n * bigintobj_bsize (args);
+  p = (bigintobjseq_t) OC_Realloc (self, m);
+#if OBJC_REFCNT
+  assert (oldc <= n);
+  memset (p + oldc, 0, (n - oldc) * bigintobj_bsize (args));
+#endif
+  return p;
 }
 
 static bigintobjseq_t 
@@ -1550,9 +1580,10 @@ bigintobjvec_iseq (bigintobjvec_t a, bigintobjvec_t b, bigintobjvec_args args)
 static void 
 bigintobjvec_expand (bigintobjvec_t self, bigintobjvec_args args)
 {
+  int oldc = self->c;
   assert (self->n == self->c);
   self->c = self->n + self->c + 1;
-  self->ptr = bigintobjseq_realloc (self->ptr, self->c, args.sargs);
+  self->ptr = bigintobjseq_realloc (self->ptr, oldc, self->c, args.sargs);
   assert (self->n < self->c);
 }
 
@@ -1940,27 +1971,35 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return &value;
 }
 
-- bigintobjvec_value:(bigintobjvec_t)aValue
+- dobigintobjvec_value:(bigintobjvec_t)aValue
 {
-  self = [self clone];
   bigintobjvec_copy (&value, aValue, bigintobjvec_getargs (self));
   assert ([self check]);
   return self;
 }
 
+- bigintobjvec_value:(bigintobjvec_t)aValue
+{
+  return [[self clone] dobigintobjvec_value:aValue];
+}
+
 - (bigintobjvec_t)bigintobjvec_reference
 {
   return &value;
 }
 
-- bigintobjvec_reference:(bigintobjvec_c *)aReference
+- dobigintobjvec_reference:(bigintobjvec_c *)aReference
 {
-  self = [self clone];
   bigintobjvec_move (&value, aReference, bigintobjvec_getargs (self));
   assert ([self check]);
   return self;
 }
 
+- bigintobjvec_reference:(bigintobjvec_c *)aReference
+{
+  return [[self clone] dobigintobjvec_reference:aReference];
+}
+
 - capacity:(int)aCapacity
 {
   bigintobjvec_c c;
@@ -2198,13 +2237,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- negateSelf
-{
-  assert ([self check]);
-  bigintobjvec_inneg (bigintobjvec_id2c (self), bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - _double:(int)v
 {
   bigintobjvec_c c;
@@ -2213,13 +2245,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- _doubleSelf:(int)v
-{
-  assert ([self check]);
-  bigintobjvec_indbl (bigintobjvec_id2c (self), v, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - double
 {
   bigintobjvec_c c;
@@ -2228,13 +2253,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- doubleSelf
-{
-  assert ([self check]);
-  bigintobjvec_indbl (bigintobjvec_id2c (self), +1, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - _add:(int)v:b:(int)w
 {
   bigintobjvec_c c;
@@ -2243,13 +2261,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- _addSelf:(int)v:b:(int)w
-{
-  assert ([self sameClass:b] && v * v == 1 && w * w == 1 && self != b);
-  bigintobjvec_inadd (bigintobjvec_id2c (self), v, bigintobjvec_id2t (b), w, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - add:b
 {
   bigintobjvec_c c;
@@ -2258,13 +2269,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- addSelf:b
-{
-  assert ([self sameClass:b] && self != b);
-  bigintobjvec_inadd (bigintobjvec_id2c (self), +1, bigintobjvec_id2t (b), +1, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - subtract:b
 {
   bigintobjvec_c c;
@@ -2273,13 +2277,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- subtractSelf:b
-{
-  assert ([self sameClass:b] && self != b);
-  bigintobjvec_inadd (bigintobjvec_id2c (self), +1, bigintobjvec_id2t (b), -1, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - multiplyScalar:s
 {
   bigintobjvec_c c;
@@ -2288,13 +2285,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- multiplySelfScalar:s
-{
-  bigintobj_c sc;
-  bigintobjvec_inmulsc (bigintobjvec_id2c (self), bigintobj_id2ref (s, &sc), bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - divideScalar:s
 {
   bigintobjvec_c c;
@@ -2303,13 +2293,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return (failed) ? nil : bigintobjvec_c2id (self, &c);
 }
 
-- divideSelfScalar:s
-{
-  bigintobj_c sc;
-  bigintobjvec_indivsc (bigintobjvec_id2c (self), bigintobj_id2ref (s, &sc), bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - _add:(int)v:B multiplyScalar:b:(int)w
 {
   bigintobjvec_c c;
@@ -2319,14 +2302,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- _addSelf:(int)v:B multiplyScalar:b:(int)w
-{
-  bigintobj_c bc;
-  assert ([self sameClass:B] && [scalarZero sameClass:b] && v * v == 1 && w * w == 1);
-  bigintobjvec_inaddmulsc (bigintobjvec_id2c (self), v, bigintobjvec_id2t (B), bigintobj_id2ref (b, &bc), w, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - _multiplyScalar:a:(int)v add:B:(int)w
 {
   bigintobjvec_c c;
@@ -2336,14 +2311,6 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- _multiplySelfScalar:a:(int)v add:B:(int)w
-{
-  bigintobj_c ac;
-  assert ([scalarZero sameClass:a] && [self sameClass:B] && v * v == 1 && w * w == 1);
-  bigintobjvec_inmulscadd (bigintobjvec_id2c (self), bigintobj_id2ref (a, &ac), v, bigintobjvec_id2t (B), w, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
-
 - _multiplyScalar:a:(int)v add:B multiplyScalar:b:(int)w
 {
   bigintobjvec_c c;
@@ -2353,12 +2320,5 @@ bigintobjvec_ix_prev (bigintobjvec_ixt ix)
   return bigintobjvec_c2id (self, &c);
 }
 
-- _multiplySelfScalar:a:(int)v add:B multiplyScalar:b:(int)w
-{
-  bigintobj_c ac, bc;
-  assert ([scalarZero sameClass:a] && [self sameClass:B] && [scalarZero sameClass:b] && v * v == 1 && w * w == 1);
-  bigintobjvec_inmulscaddmulsc (bigintobjvec_id2c (self), bigintobj_id2ref (a, &ac), v, bigintobjvec_id2t (B), bigintobj_id2ref (b, &bc), w, bigintobjvec_getargs (self));
-  return [self invalidate];
-}
 @end
  
