@@ -1,7 +1,23 @@
 
 /*
- * Computer Algebra Kit (c) 1993,99 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: matrix.m,v 1.2 1999/07/06 19:48:10 stes Exp $
+ * Computer Algebra Kit (c) 1993,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: matrix.m,v 1.6 2000/10/12 14:40:26 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "cakit.h"
@@ -33,59 +49,63 @@
 
 + diagonal:cltnOfScalars
 {
+  id c;
   int i, n = [cltnOfScalars size];
   if (n == 0)
     [self error:"Collection Must Not Be Empty"];
 
-  self = [self scalarZero:[[cltnOfScalars lastElement] zero] numRows:n numColumns:n];
+  c = [self scalarZero:[[cltnOfScalars lastElement] zero] numRows:n numColumns:n];
   for (i = 0; i < n; i++)
-    [self placeScalar:[cltnOfScalars at:i] at:i:i];
+    [c placeScalar:[cltnOfScalars at:i] at:i:i];
 
-  assert ([self isDiagonal]);
-  return self;
+  assert ([c isDiagonal]);
+  return c;
 }
 
 + circulant:cltnOfScalars
 {
+  id c;
   int i, j, n = [cltnOfScalars size];
   if (n == 0)
     [self error:"Collection Must Not Be Empty"];
 
-  self = [self scalarZero:[[cltnOfScalars lastElement] zero] numRows:n numColumns:n];
+  c = [self scalarZero:[[cltnOfScalars lastElement] zero] numRows:n numColumns:n];
   for (i = 0; i < n; i++)
     {
       for (j = 0; j < n; j++)
 	{
-	  [self placeScalar:[cltnOfScalars at:(n - i + j) % n] at:i:j];
+	  [c placeScalar:[cltnOfScalars at:(n - i + j) % n] at:i:j];
 	}
     }
-  return self;
+  return c;
 }
 
 + companion:cltnOfScalars
 {
+  id c;
   id aZero;
   int i, n = [cltnOfScalars size];
   if (n == 0)
     [self error:"Collection Must Not Be Empty"];
 
   aZero = [[cltnOfScalars lastElement] zero];
-  self = [self scalarZero:aZero numRows:n numColumns:n];
+  c = [self scalarZero:aZero numRows:n numColumns:n];
   for (i = 0; i < n - 1; i++)
-    [self placeScalar:[aZero one] at:i + 1:i];
+    [c placeScalar:[aZero one] at:i + 1:i];
   for (i = 0; i < n; i++)
-    [self placeScalar:[[cltnOfScalars at:i] negate] at:i:n - 1];
-  return self;
+    [c placeScalar:[[cltnOfScalars at:i] negate] at:i:n - 1];
+  return c;
 }
 
 + hankel:rowScalars:colScalars
 {
+  id c;
   id obj;
   int i, j, k, n, m;
 
   n = [colScalars size] + 1;
   m = [rowScalars size];
-  self = [self scalarZero:[[rowScalars lastElement] zero] numRows:n numColumns:m];
+  c = [self scalarZero:[[rowScalars lastElement] zero] numRows:n numColumns:m];
 
   for (i = 0; i < n; i++)
     {
@@ -93,52 +113,54 @@
 	{
 	  k = i + j;
 	  obj = (k < m) ? [rowScalars at : k]:[colScalars at:k - m];
-	  [self placeScalar:obj at:i:j];
+	  [c placeScalar:obj at:i:j];
 	}
     }
 
-  return self;
+  return c;
 }
 
 + toeplitz:rowScalars:colScalars
 {
+  id c;
   id obj;
   int i, j, k, n, m;
 
   n = [colScalars size];
   m = [rowScalars size] + 1;
-  self = [self scalarZero:[[rowScalars lastElement] zero] numRows:n numColumns:m];
+  c = [self scalarZero:[[rowScalars lastElement] zero] numRows:n numColumns:m];
 
   for (i = 0; i < n; i++)
     for (j = 0; j < m; j++)
       {
 	k = j - i + n - 1;
 	obj = (k < n) ? [colScalars at : k]:[rowScalars at:k - n];
-	[self placeScalar:obj at:i:j];
+	[c placeScalar:obj at:i:j];
       }
 
-  return self;
+  return c;
 }
 
 + hilbert:(int)n scalarZero:aZero
 {
+  id c;
   id obj;
   int i, j, k;
-  self = [self scalarZero:aZero numRows:n numColumns:n];
+  c = [self scalarZero:aZero numRows:n numColumns:n];
 
   for (k = 1; k < (n + n); k++)
     {
       obj = [aZero intValue:k];
-      obj = [obj inverseSelf];
+      obj = [obj inverse];
       for (i = 0; i < n; i++)
 	for (j = 0; j < n; j++)
 	  {
 	    if (i + j + 1 == k)
-	      [self placeScalar:obj at:i:j];
+	      [c placeScalar:obj at:i:j];
 	  }
     }
 
-  return self;
+  return c;
 }
 
 + hilbert:(int)n
@@ -182,23 +204,26 @@
 
 + scalar:aScalar numRows:(int)numRows numColumns:(int)numColumns
 {
+  id c;
   int i = (numRows < numColumns) ? numRows : numColumns;
-  self = [self scalarZero:[aScalar zero] numRows:numRows numColumns:numColumns];
+  c = [self scalarZero:[aScalar zero] numRows:numRows numColumns:numColumns];
   while (i--)
-    [self placeScalar:aScalar at:i:i];
-  return self;
+    [c placeScalar:aScalar at:i:i];
+  return c;
 }
-- copy
+- setrows:r
 {
-  self = [super copy];
-  rows = [rows deepCopy];
-  assert ([self check]);
+  rows = r;
   return self;
 }
 
-- deepCopy
+- copy
+{
+  return [[super copy] setrows:[rows deepCopy]];
+}
+
+- dodeepCopy
 {
-  self = [super deepCopy];
   scalarZero = [scalarZero deepCopy];
   emptyVector = [emptyVector deepCopy];
   rows = [rows deepCopy];
@@ -206,16 +231,25 @@
   return self;
 }
 
+- deepCopy
+{
+  return [[super deepCopy] dodeepCopy];
+}
+
 - clone
 {
-  self = [super clone];
-  rows = nil;
-  return self;
+  return [[super clone] setrows:nil];
 }
 
 - rows:c
 {
-  self = [self clone];
+  return [[super clone] setrows:c];
+}
+
+- doscalarZero:aZero emptyVector:aVector rows:c
+{
+  scalarZero = aZero;
+  emptyVector = aVector;
   rows = c;
   assert ([self check]);
   return self;
@@ -223,12 +257,7 @@
 
 - scalarZero:aZero emptyVector:aVector rows:c
 {
-  self = [super clone];
-  scalarZero = aZero;
-  emptyVector = aVector;
-  rows = c;
-  assert ([self check]);
-  return self;
+  return [[super clone] doscalarZero:aZero emptyVector:aVector rows:c];
 }
 
 - numRows:(int)numRows numColumns:(int)numColumns
@@ -788,12 +817,13 @@
 
 - one
 {
+  id c;
   int m = [self numRows];
   int n = [self numColumns];
-  self = [self zero];
+  c = [self zero];
   n = (n < m) ? n : m;
   while (n--)
-    [self placeScalar:[scalarZero one] at:n:n];
+    [c placeScalar:[scalarZero one] at:n:n];
   return self;
 }
 
@@ -884,10 +914,6 @@
 
   return c;
 }
-- transposeSelf
-{
-  return [self transpose];
-}
 
 - determinant
 {
@@ -902,7 +928,7 @@
     {
       id n, d, m;
       m = [self onCommonDenominator:&d];
-      d = [d powerSelf:[self numRows]];
+      d = [d power:[self numRows]];
       n = [m determinant];
       return [scalarZero numerator:n denominator:d];
     }
@@ -962,8 +988,8 @@
 	  [rn swapElementsAt:i:j];
 	}
 
-      det = [det multiplySelf:p];
-      p = [p inverseSelf];
+      det = [det multiply:p];
+      p = [p inverse];
       assert (p);
 
       /* reduction step - modify behind back of "rn" member at "j"
@@ -974,15 +1000,15 @@
 	{
 	  rj = [rn at:j];
 	  q = [rj removeScalar];
-	  q = [q multiplySelf:p];
-	  rj = [rj subtractSelf:ri multiplyScalar:q];
+	  q = [q multiply:p];
+	  rj = [rj subtract:ri multiplyScalar:q];
 	  [rn at:j put:rj];
 	}
 
       assert ([rn check]);
     }
 
-  det = (sign > 0) ? det : [det negateSelf];
+  det = (sign > 0) ? det : [det negate];
   return det;
 }
 
@@ -1029,7 +1055,7 @@
 	  for (k = i; k < j; k++)
 	    {
 	      id rk = [rn at:k];
-	      rk = [rk multiplySelfScalar:p divideScalar:det];
+	      rk = [rk multiplyScalar:p divideScalar:det];
 	      [rn at:k put:rk];
 	    }
 	  [rn swapElementsAt:i:j];
@@ -1044,7 +1070,7 @@
 	{
 	  rj = [rn at:j];
 	  q = [rj removeScalar];
-	  rj = [rj multiplySelfScalar:p subtract:ri multiplyScalar:q divideScalar:det];
+	  rj = [rj multiplyScalar:p subtract:ri multiplyScalar:q divideScalar:det];
 	  [rn at:j put:rj];
 	}
 
@@ -1052,7 +1078,7 @@
       det = p;
     }
 
-  det = (sign > 0) ? det : [det negateSelf];
+  det = (sign > 0) ? det : [det negate];
   return det;
 }
 
@@ -1088,7 +1114,7 @@
 	{
 	  id det, x;
 	  x = [matrix solveBareissDeterminant:&det];
-	  x = [x divideSelfScalar:det];
+	  x = [x divideScalar:det];
 	  return x;
 	}
     }
@@ -1128,24 +1154,24 @@
 	}
 
       if (detReference)
-	det = [det multiplySelf:p];
-      p = [p inverseSelf];
+	det = [det multiply:p];
+      p = [p inverse];
       assert (p);
-      ri = [ri multiplySelfScalar:p];
+      ri = [ri multiplyScalar:p];
       [rn at:i put:ri];
 
       for (j++; j < n; j++)
 	{
 	  rj = [rn at:j];
 	  q = [rj removeScalar];
-	  rj = [rj subtractSelf:ri multiplyScalar:q];
+	  rj = [rj subtract:ri multiplyScalar:q];
 	  [rn at:j put:rj];
 	}
 
       assert ([rn check]);
     }
   if (detReference)
-    det = (sign > 0) ? det : [det negateSelf];
+    det = (sign > 0) ? det : [det negate];
 
   x = [emptyVector capacity:n + 1];
   x = [x insertScalar:[scalarZero minusOne]];
@@ -1153,7 +1179,7 @@
   while (i--)
     {
       q = [x dotMultiply:[rn at:i]];
-      [x insertScalar:[q negateSelf]];
+      [x insertScalar:[q negate]];
     }
 
   if (detReference)
@@ -1194,7 +1220,7 @@
 	  int k;
 	  for (k = i; k < j; k++)
 	    {
-	      rk = [[rn at:k] multiplySelfScalar:p divideScalar:det];
+	      rk = [[rn at:k] multiplyScalar:p divideScalar:det];
 	      [rn at:k put:rk];
 	    }
 	  sign = -sign;
@@ -1205,7 +1231,7 @@
 	{
 	  rj = [rn at:j];
 	  q = [rj removeScalar];
-	  rj = [rj multiplySelfScalar:p subtract:ri multiplyScalar:q divideScalar:det];
+	  rj = [rj multiplyScalar:p subtract:ri multiplyScalar:q divideScalar:det];
 	  [rn at:j put:rj];
 	}
 
@@ -1214,28 +1240,28 @@
       det = p;
     }
 
-  det = (sign > 0) ? det : [det negateSelf];
+  det = (sign > 0) ? det : [det negate];
 
   x = [emptyVector capacity:n + 1];
   p = [piv removeLast];
   if (sign > 0)
     {
       assert ([p isEqual:det]);
-      x = [x insertScalar:[p negateSelf]];
+      x = [x insertScalar:[p negate]];
       x = [x insertScalar:[[rn at:n - 1] removeScalar]];
     }
   else
     {
       assert ([p isOpposite:det]);
       x = [x insertScalar:p];
-      x = [x insertScalar:[[[rn at:n - 1] removeScalar] negateSelf]];
+      x = [x insertScalar:[[[rn at:n - 1] removeScalar] negate]];
     }
   i = n - 1;
   while (i--)
     {
       q = [x dotMultiply:[rn at:i]];
       p = [piv removeLast];
-      q = [[q divideSelf:p] negateSelf];
+      q = [[q divide:p] negate];
       [x insertScalar:q];
     }
 
@@ -1248,7 +1274,7 @@
 - inverse
 {
   [self checkSquare];
-  return [[self one] divideSelf:self];
+  return [[self one] divide:self];
 }
 
 - divide:b
@@ -1303,11 +1329,11 @@
 	  [irn swapElementsAt:i:j];
 	}
 
-      p = [p inverseSelf];
+      p = [p inverse];
       assert (p);
       iri = [irn at:i];
-      ri = [ri multiplySelfScalar:p];
-      iri = [iri multiplySelfScalar:p];
+      ri = [ri multiplyScalar:p];
+      iri = [iri multiplyScalar:p];
       [rn at:i put:ri];
       [irn at:i put:iri];
 
@@ -1315,10 +1341,10 @@
 	{
 	  rj = [rn at:j];
 	  q = [rj removeScalar];
-	  rj = [rj subtractSelf:ri multiplyScalar:q];
+	  rj = [rj subtract:ri multiplyScalar:q];
 	  [rn at:j put:rj];
 	  irj = [irn at:j];
-	  irj = [irj subtractSelf:iri multiplyScalar:q];
+	  irj = [irj subtract:iri multiplyScalar:q];
 	  [irn at:j put:irj];
 	}
 
@@ -1335,7 +1361,7 @@
 	{
 	  q = [t removeLast];
 	  irj = [irn at:j];
-	  irj = [irj subtractSelf:iri multiplyScalar:q];
+	  irj = [irj subtract:iri multiplyScalar:q];
 	  [irn at:j put:irj];
 	}
     }
@@ -1379,23 +1405,23 @@
 	{
 	  rowmark [i] = YES;
 
-	  q = [q inverseSelf];
+	  q = [q inverse];
 	  assert (q);
-	  ri = [ri multiplySelfScalar:q];
+	  ri = [ri multiplyScalar:q];
 	  [rm at:i put:ri];
 
 	  for (k = i - 1; k >= 0; k--)
 	    {
 	      q = [qj removeLast];
 	      rk = [rm at:k];
-	      [rm at:k put:[rk subtractSelf:ri multiplyScalar:q]];
+	      [rm at:k put:[rk subtract:ri multiplyScalar:q]];
 	    }
 
 	  for (k = i + 1; k < m; k++)
 	    {
 	      rk = [rm at:k];
 	      q = [rk removeScalar];
-	      [rm at:k put:[rk subtractSelf:ri multiplyScalar:q]];
+	      [rm at:k put:[rk subtract:ri multiplyScalar:q]];
 	    }
 	}
 
@@ -1455,16 +1481,16 @@
 	  indexrank [j] = i;
 	  rowmark [i] = YES;
 
-	  q = [q inverseSelf];
+	  q = [q inverse];
 	  assert (q);
-	  ri = [ri multiplySelfScalar:q];
+	  ri = [ri multiplyScalar:q];
 	  [rm at:i put:ri];
 	  [qj add:[scalarZero one]];
 
 	  for (k = 0; k < i; k++)
 	    {
 	      q = [qj at:k put:[scalarZero zero]];
-	      rk = [[rm at:k] subtractSelf:ri multiplyScalar:q];
+	      rk = [[rm at:k] subtract:ri multiplyScalar:q];
 	      [rm at:k put:rk];
 	    }
 
@@ -1472,7 +1498,7 @@
 	    {
 	      rk = [rm at:k];
 	      q = [rk removeScalar];
-	      rk = [rk subtractSelf:ri multiplyScalar:q];
+	      rk = [rk subtract:ri multiplyScalar:q];
 	      [rm at:k put:rk];
 	      [qj add:[scalarZero zero]];
 	    }
@@ -1561,7 +1587,7 @@
 
   seq = [self eachSequence];
   while (n--)
-    trace = [trace addSelf:[[seq at:n] at:n]];
+    trace = [trace add:[[seq at:n] at:n]];
 
   return trace;
 }
@@ -1592,11 +1618,6 @@
   return adj;
 }
 
-- adjointSelf
-{
-  return [self adjoint];
-}
-
 
 - printOn:(IOD)aFile
 {
