@@ -543,8 +543,24 @@ modp_fileIn (id aFiler, modp_c * c, modp_args args)
 
 
 /*
- * Computer Algebra Kit (c) 1993,98 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: mdrdpolc.m,v 1.2 1999/07/06 19:48:10 stes Exp $
+ * Computer Algebra Kit (c) 1993,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: mdrdpolc.m,v 1.6 2000/10/12 14:40:26 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static int 
@@ -728,13 +744,27 @@ modpseq_check (modpseq_t self, int n, modp_args args)
 static modpseq_t 
 modpseq_alloc (int n, modp_args args)
 {
-  return (modpseq_t) OC_Malloc (n * modp_bsize (args));
+  modpseq_t p;
+  int m = n * modp_bsize (args);
+#if OBJC_REFCNT
+  p = (modpseq_t) OC_Calloc (m);
+#else
+  p = (modpseq_t) OC_Malloc (m);
+#endif
+  return p;
 }
 
 static modpseq_t 
-modpseq_realloc (modpseq_t self, int n, modp_args args)
+modpseq_realloc (modpseq_t self, int oldc, int n, modp_args args)
 {
-  return (modpseq_t) OC_Realloc (self, n * modp_bsize (args));
+  modpseq_t p;
+  int m = n * modp_bsize (args);
+  p = (modpseq_t) OC_Realloc (self, m);
+#if OBJC_REFCNT
+  assert (oldc <= n);
+  memset (p + oldc, 0, (n - oldc) * modp_bsize (args));
+#endif
+  return p;
 }
 
 static modpseq_t 
@@ -1420,14 +1450,14 @@ modp_vardnsrecdegdnspolseq_inneg (modp_vardnsrecdegdnspolseq_t a, int n, modp_va
 static int 
 modp_vardnsrecdegdnspolseq_oodbl (modp_vardnsrecdegdnspolseq_t c, modp_vardnsrecdegdnspolseq_t a, int n, int v, modp_vardnsrecdegdnspol_args args)
 {
-  modpseq_oodbl (c, a, n, v, args.sargs);
+  modpseq_oodbl (c, a, v, n, args.sargs);
   return modp_vardnsrecdegdnspolseq_removezeroes (c, n, args);
 }
 
 static int 
 modp_vardnsrecdegdnspolseq_indbl (modp_vardnsrecdegdnspolseq_t a, int n, int v, modp_vardnsrecdegdnspol_args args)
 {
-  modpseq_indbl (a, n, v, args.sargs);
+  modpseq_indbl (a, v, n, args.sargs);
   return modp_vardnsrecdegdnspolseq_removezeroes (a, n, args);
 }
 
@@ -1745,7 +1775,7 @@ modp_vardnsrecdegdnspolseq_ooaddmulmonomsc (modp_vardnsrecdegdnspolseq_t c, modp
 	      c += an * modp_size (args.sargs);
 	      a += an * modp_size (args.sargs);
 	      modpseq_oozero (c, m - an, args.sargs);
-	      c += m * modp_size (args.sargs);
+	      c += (m - an) * modp_size (args.sargs);
 	      modpseq_oomulscv (c, b, t, w, bn, args.sargs);
 	    }
 	  else
@@ -1757,7 +1787,7 @@ modp_vardnsrecdegdnspolseq_ooaddmulmonomsc (modp_vardnsrecdegdnspolseq_t c, modp
 	      assert (bn > an - m);
 	      modpseq_ooaddmulsc (c, a, v, b, t, w, an - m, args.sargs);
 	      c += (an - m) * modp_size (args.sargs);
-	      a += (an - m) * modp_size (args.sargs);
+	      b += (an - m) * modp_size (args.sargs);
 	      modpseq_oomulscv (c, b, t, w, bn - (an - m), args.sargs);
 	    }
 	  return bn + m;
@@ -1803,7 +1833,7 @@ modp_vardnsrecdegdnspolseq_inaddmulmonomsc (modp_vardnsrecdegdnspolseq_t a, int
 	      modpseq_innegv (a, v, an, args.sargs);
 	      a += an * modp_size (args.sargs);
 	      modpseq_oozero (a, m - an, args.sargs);
-	      a += m * modp_size (args.sargs);
+	      a += (m - an) * modp_size (args.sargs);
 	      modpseq_oomulscv (a, b, t, w, bn, args.sargs);
 	    }
 	  else
@@ -1859,7 +1889,7 @@ modp_vardnsrecdegdnspolseq_oomulscaddmulmonomsc (modp_vardnsrecdegdnspolseq_t c,
 	      c += an * modp_size (args.sargs);
 	      a += an * modp_size (args.sargs);
 	      modpseq_oozero (c, m - an, args.sargs);
-	      c += m * modp_size (args.sargs);
+	      c += (m - an) * modp_size (args.sargs);
 	      modpseq_oomulscv (c, b, t, w, bn, args.sargs);
 	    }
 	  else
@@ -1872,6 +1902,7 @@ modp_vardnsrecdegdnspolseq_oomulscaddmulmonomsc (modp_vardnsrecdegdnspolseq_t c,
 	      modpseq_oomulscaddmulsc (c, a, s, v, b, t, w, an - m, args.sargs);
 	      c += (an - m) * modp_size (args.sargs);
 	      a += (an - m) * modp_size (args.sargs);
+	      b += (an - m) * modp_size (args.sargs);
 	      modpseq_oomulscv (c, b, t, w, bn - (an - m), args.sargs);
 	    }
 	  return bn + m;
@@ -1946,6 +1977,7 @@ modp_vardnsrecdegdnspolseq_inmulscaddmulmonomsc (modp_vardnsrecdegdnspolseq_t a,
 	}
     }
 }
+
 static void 
 modp_vardnsrecdegdnspol_ix_init (modp_vardnsrecdegdnspol_ixt ix, id self)
 {
@@ -2209,8 +2241,9 @@ modp_vardnsrecdegdnspol_expfirst (modp_vardnsrecdegdnspol_t self, modp_vardnsrec
 static void 
 modp_vardnsrecdegdnspol_expand (modp_vardnsrecdegdnspol_t self, modp_vardnsrecdegdnspol_args args)
 {
+  int oldc = self->c;
   self->c = self->n + self->c + 1;
-  self->ptr = modpseq_realloc (self->ptr, self->c, args.sargs);
+  self->ptr = modpseq_realloc (self->ptr, oldc, self->c, args.sargs);
   assert (self->n < self->c);
 }
 
@@ -2513,7 +2546,7 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   id aSymbol;
 
   assert ([aCoef isZero] && [aScalar isZero]);
-  assert ([aCoef class_vardnsrecdegdns_polynomial] == self);
+  assert ([aCoef class_vardnsrecdegdns_polynomial] == [self class]);
 
   aSymbol = [aCltn lastElement];
   termZero = [Term scalarZero:aScalar coefficient:aCoef symbol:aSymbol exponent:1];
@@ -2551,27 +2584,35 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return &value;
 }
 
-- modp_vardnsrecdegdnspol_value:(modp_vardnsrecdegdnspol_t)aValue
+- domodp_vardnsrecdegdnspol_value:(modp_vardnsrecdegdnspol_t)aValue
 {
-  self = [self clone];
   modp_vardnsrecdegdnspol_copy (&value, aValue, modp_vardnsrecdegdnspol_getargs (self));
   assert ([self check]);
   return self;
 }
 
+- modp_vardnsrecdegdnspol_value:(modp_vardnsrecdegdnspol_t)aValue
+{
+  return [[self clone] domodp_vardnsrecdegdnspol_value:aValue];
+}
+
 - (modp_vardnsrecdegdnspol_t)modp_vardnsrecdegdnspol_reference
 {
   return &value;
 }
 
-- modp_vardnsrecdegdnspol_reference:(modp_vardnsrecdegdnspol_c *)aReference
+- domodp_vardnsrecdegdnspol_reference:(modp_vardnsrecdegdnspol_c *)aReference
 {
-  self = [self clone];
   modp_vardnsrecdegdnspol_move (&value, aReference, modp_vardnsrecdegdnspol_getargs (self));
   assert ([self check]);
   return self;
 }
 
+- modp_vardnsrecdegdnspol_reference:(modp_vardnsrecdegdnspol_c *)aReference
+{
+  return [[self clone] domodp_vardnsrecdegdnspol_reference:aReference];
+}
+
 - copy
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2666,13 +2707,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- negateSelf
-{
-  assert ([self check]);
-  modp_vardnsrecdegdnspol_inneg (modp_vardnsrecdegdnspol_id2c (self), modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _double:(int)v
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2681,13 +2715,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _doubleSelf:(int)v
-{
-  assert ([self check]);
-  modp_vardnsrecdegdnspol_indbl (modp_vardnsrecdegdnspol_id2c (self), v, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - double
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2696,13 +2723,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- doubleSelf
-{
-  assert ([self check]);
-  modp_vardnsrecdegdnspol_indbl (modp_vardnsrecdegdnspol_id2c (self), +1, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _add:(int)v:b:(int)w
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2711,13 +2731,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _addSelf:(int)v:b:(int)w
-{
-  assert ([self sameClass:b] && v * v == 1 && w * w == 1 && self != b);
-  modp_vardnsrecdegdnspol_inadd (modp_vardnsrecdegdnspol_id2c (self), v, modp_vardnsrecdegdnspol_id2t (b), w, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - add:b
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2726,13 +2739,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- addSelf:b
-{
-  assert ([self sameClass:b] && self != b);
-  modp_vardnsrecdegdnspol_inadd (modp_vardnsrecdegdnspol_id2c (self), +1, modp_vardnsrecdegdnspol_id2t (b), +1, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - subtract:b
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2741,13 +2747,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- subtractSelf:b
-{
-  assert ([self sameClass:b] && self != b);
-  modp_vardnsrecdegdnspol_inadd (modp_vardnsrecdegdnspol_id2c (self), +1, modp_vardnsrecdegdnspol_id2t (b), -1, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - (int) leadingDegree
 {
   return modp_vardnsrecdegdnspol_deg (&value, modp_vardnsrecdegdnspol_getargs (self));
@@ -2918,13 +2917,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- multiplySelfCoefficient:s
-{
-  modp_c sc;
-  modp_vardnsrecdegdnspol_inmulsc (modp_vardnsrecdegdnspol_id2c (self), modp_id2ref (s, &sc), modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - divideCoefficient:s
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2933,13 +2925,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return (failed) ? nil : modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- divideSelfCoefficient:s
-{
-  modp_c sc;
-  modp_vardnsrecdegdnspol_indivsc (modp_vardnsrecdegdnspol_id2c (self), modp_id2ref (s, &sc), modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _add:(int)v:B multiplyCoefficient:b:(int)w
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2948,13 +2933,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _addSelf:(int)v:B multiplyCoefficient:b:(int)w
-{
-  modp_c bc;
-  modp_vardnsrecdegdnspol_inaddmulsc (modp_vardnsrecdegdnspol_id2c (self), v, modp_vardnsrecdegdnspol_id2t (B), modp_id2ref (b, &bc), w, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _multiplyCoefficient:a:(int)v add:B:(int)w
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2963,13 +2941,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _multiplySelfCoefficient:a:(int)v add:B:(int)w
-{
-  modp_c ac;
-  modp_vardnsrecdegdnspol_inmulscadd (modp_vardnsrecdegdnspol_id2c (self), modp_id2ref (a, &ac), v, modp_vardnsrecdegdnspol_id2t (B), w, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _multiplyCoefficient:a:(int)v add:B multiplyCoefficient:b:(int)w
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2978,12 +2949,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _multiplySelfCoefficient:a:(int)v add:B multiplyCoefficient:b:(int)w
-{
-  modp_c ac, bc;
-  modp_vardnsrecdegdnspol_inmulscaddmulsc (modp_vardnsrecdegdnspol_id2c (self), modp_id2ref (a, &ac), v, modp_vardnsrecdegdnspol_id2t (B), modp_id2ref (b, &bc), w, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
 - multiplyTerm:aTerm
 {
   modp_vardnsrecdegdnspol_c c;
@@ -2994,15 +2959,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- multiplySelfTerm:aTerm
-{
-  modp_t s;
-  modp_c sc;
-  s = modp_id2ref ([aTerm coefficient], &sc);
-  modp_vardnsrecdegdnspol_inmulmonomsc (modp_vardnsrecdegdnspol_id2c (self), [aTerm exponent], s, +1, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _multiplyTerm:aTerm:(int)v
 {
   modp_vardnsrecdegdnspol_c c;
@@ -3013,15 +2969,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _multiplySelfTerm:aTerm:(int)v
-{
-  modp_t s;
-  modp_c sc;
-  s = modp_id2ref ([aTerm coefficient], &sc);
-  modp_vardnsrecdegdnspol_inmulmonomsc (modp_vardnsrecdegdnspol_id2c (self), [aTerm exponent], s, v, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - divideTerm:aTerm
 {
   int e;
@@ -3036,15 +2983,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return (failed) ? nil : modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- divideSelfTerm:aTerm
-{
-  modp_t s;
-  modp_c sc;
-  s = modp_id2ref ([aTerm coefficient], &sc);
-  modp_vardnsrecdegdnspol_indivmonomsc (modp_vardnsrecdegdnspol_id2c (self), [aTerm exponent], s, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _add:(int)v:B multiplyTerm:b:(int)w
 {
   modp_vardnsrecdegdnspol_c c;
@@ -3055,15 +2993,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _addSelf:(int)v:B multiplyTerm:b:(int)w
-{
-  modp_c bc;
-  modp_t bs;
-  bs = modp_id2ref ([b coefficient], &bc);
-  modp_vardnsrecdegdnspol_inaddmulmonomsc (modp_vardnsrecdegdnspol_id2c (self), v, modp_vardnsrecdegdnspol_id2t (B), [b exponent], bs, w, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
-
 - _multiplyCoefficient:a:(int)v add:B multiplyTerm:b:(int)w
 {
   modp_vardnsrecdegdnspol_c c;
@@ -3075,15 +3004,6 @@ modp_vardnsrecdegdnspol_fileIn (id aFiler, modp_vardnsrecdegdnspol_t self, modp_
   return modp_vardnsrecdegdnspol_c2id (self, &c);
 }
 
-- _multiplySelfCoefficient:a:(int)v add:B multiplyTerm:b:(int)w
-{
-  modp_c ac, bc;
-  modp_t as, bs;
-  as = modp_id2ref (a, &ac);
-  bs = modp_id2ref ([b coefficient], &bc);
-  modp_vardnsrecdegdnspol_inmulscaddmulmonomsc (modp_vardnsrecdegdnspol_id2c (self), as, v, modp_vardnsrecdegdnspol_id2t (B), [b exponent], bs, w, modp_vardnsrecdegdnspol_getargs (self));
-  return [self invalidate];
-}
 
 - fileOutOn:aFiler
 {
