@@ -1,7 +1,23 @@
 
 /*
- * Computer Algebra Kit (c) 1993,99 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: polynom.m,v 1.2 1999/07/06 19:48:12 stes Exp $
+ * Computer Algebra Kit (c) 1993,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: polynom.m,v 1.7 2000/10/12 14:40:26 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "cakit.h"
@@ -82,9 +98,8 @@
   return [super invalidate];
 }
 
-- clone
+- clearpoly
 {
-  self = [super clone];
   carrier = nil;
   _leadingMonomial = nil;
   _leadingTerm = nil;
@@ -93,29 +108,35 @@
   return self;
 }
 
+- clone
+{
+  return [[super clone] clearpoly];
+}
+
 - carrier
 {
   return carrier;
 }
 
-- over:c
+- setcarrier:c
 {
-  assert ([c sameClass:carrier]);
-  self = [self clone];
   carrier = c;
   assert ([self check]);
   return self;
 }
 
+- over:c
+{
+  return [[self clone] setcarrier:c];
+}
+
 - empty
 {
   return [self over:[carrier empty]];
 }
 
-- differentCarrier:c
+- dodifferentcarrier:c
 {
-  self = [super clone];
-
   scalarZero = [c scalarZero];
   termZero = [c termZero];
   monomialZero = [c monomialZero];
@@ -132,6 +153,11 @@
   return self;
 }
 
+- differentCarrier:c
+{
+  return [[super clone] dodifferentcarrier:c];
+}
+
 - emptyScalarZero:aZero
 {
   id res = [self differentCarrier:[carrier emptyScalarZero:aZero]];
@@ -915,7 +941,7 @@
   while ((member = [seq next]))
     {
       id add = [member changeRepresentationTo:aPrototype];
-      new = [new addSelf:add];
+      new = [new add:add];
     }
   return new;
 }
@@ -1157,25 +1183,6 @@
     }
 }
 
-- makeSelfMonic
-{
-  if ([self isMonic])
-    {
-      return self;
-    }
-  else
-    {
-      if ([self isRecursive])
-	{
-	  return [self divideSelfCoefficient:[self leadingCoefficient]];
-	}
-      else
-	{
-	  return [self divideSelfScalar:[self leadingScalar]];
-	}
-    }
-}
-
 
 - zero
 {
@@ -1231,19 +1238,6 @@
     }
 }
 
-- negateSelf
-{
-  if ([self isZero] || [self isCharacteristicTwo])
-    {
-      return self;
-    }
-  else
-    {
-      carrier = [carrier negateSelf];
-      return [self invalidate];
-    }
-}
-
 - _double:(int)v
 {
   if ([self isZero] || [self isCharacteristicTwo])
@@ -1256,19 +1250,6 @@
     }
 }
 
-- _doubleSelf:(int)v
-{
-  if ([self isZero] || [self isCharacteristicTwo])
-    {
-      return [self zero];
-    }
-  else
-    {
-      carrier = [carrier _doubleSelf:v];
-      return [self invalidate];
-    }
-}
-
 - double
 {
   if ([self isZero] || [self isCharacteristicTwo])
@@ -1281,19 +1262,6 @@
     }
 }
 
-- doubleSelf
-{
-  if ([self isZero] || [self isCharacteristicTwo])
-    {
-      return [self zero];
-    }
-  else
-    {
-      carrier = [carrier doubleSelf];
-      return [self invalidate];
-    }
-}
-
 - _add:(int)v:b:(int)w
 {
   assert ([self sameClass:b] && v * v == 1 && w * w == 1);
@@ -1315,28 +1283,6 @@
     }
 }
 
-- _addSelf:(int)v:b:(int)w
-{
-  assert ([self sameClass:b] && v * v == 1 && w * w == 1);
-
-  if (self == b)
-    {
-      return (v == w) ? [self _doubleSelf : v]:[self zero];
-    }
-  else
-    {
-      if ([self inAdditiveMonoid])
-	{
-	  if ([b isZero])
-	    return [self _negateSelf:v];
-	  if ([self isZero])
-	    return [b _negate:w];
-	}
-      carrier = [carrier _addSelf:v:[b carrier]:w];
-      return [self invalidate];
-    }
-}
-
 - add:b
 {
   [self checkSameClass:b];
@@ -1358,28 +1304,6 @@
     }
 }
 
-- addSelf:b
-{
-  assert ([self sameClass:b]);
-
-  if (self == b)
-    {
-      return [self doubleSelf];
-    }
-  else
-    {
-      if ([self inAdditiveMonoid])
-	{
-	  if ([b isZero])
-	    return self;
-	  if ([self isZero])
-	    return b;
-	}
-      carrier = [carrier addSelf:[b carrier]];
-      return [self invalidate];
-    }
-}
-
 - subtract:b
 {
   [self checkSameClass:b];
@@ -1401,28 +1325,6 @@
     }
 }
 
-- subtractSelf:b
-{
-  assert ([self sameClass:b]);
-
-  if (self == b)
-    {
-      return [self zero];
-    }
-  else
-    {
-      if ([self inAdditiveMonoid])
-	{
-	  if ([b isZero])
-	    return self;
-	  if ([self isZero])
-	    return [b negate];
-	}
-      carrier = [carrier subtractSelf:[b carrier]];
-      return [self invalidate];
-    }
-}
-
 - addScalar:s
 {
   id res, tmp = [self scalar:s];
@@ -1460,6 +1362,7 @@
 
 - multiply:b
 {
+  id a;
   [self checkSameClass:b];
 
   if (self == b || [self isEqual:b])
@@ -1482,12 +1385,15 @@
       /* outer loop of mul iterates over polynomial with smallest num terms */
       if ([self numTerms] > [b numTerms])
 	{
-	  id t = self;
-	  self = b;
-	  b = t;
+	  a = b;
+	  b = self;
+	}
+      else
+	{
+	  a = self;
 	}
 
-      if ((term = [self asTerm]))
+      if ((term = [a asTerm]))
 	{
 	  id p = [b multiplyTerm:term];
 	  return p;
@@ -1496,11 +1402,11 @@
 	{
 	  id p, seq;
 	  assert ([b asTerm] == nil);
-	  p = [self empty];
-	  seq = [self eachTerm];
+	  p = [a empty];
+	  seq = [a eachTerm];
 	  while ((term = [seq next]))
 	    {
-	      p = [p addSelf:b multiplyTerm:term];
+	      p = [p add:b multiplyTerm:term];
 	    }
 	  return p;
 	}
@@ -1512,12 +1418,15 @@
       /* outer loop of mul iterates over polynomial with smallest num terms */
       if ([self numMonomials] > [b numMonomials])
 	{
-	  id t = self;
-	  self = b;
-	  b = t;
+	  a = b;
+	  b = self;
+	}
+      else
+	{
+	  a = self;
 	}
 
-      if (monomial = [self asMonomial])
+      if (monomial = [a asMonomial])
 	{
 	  id p = [b multiplyMonomial:monomial];
 	  return p;
@@ -1526,11 +1435,11 @@
 	{
 	  id p, seq;
 	  assert ([b asMonomial] == nil);
-	  p = [self empty];
-	  seq = [self eachMonomial];
+	  p = [a empty];
+	  seq = [a eachMonomial];
 	  while ((monomial = [seq next]))
 	    {
-	      p = [p addSelf:b multiplyMonomial:monomial];
+	      p = [p add:b multiplyMonomial:monomial];
 	    }
 	  return p;
 	}
@@ -1539,45 +1448,6 @@
   return nil;
 }
 
-- multiplySelf:b
-{
-  [self checkSameClass:b];
-
-  if (self == b || [self isEqual:b])
-    return [self squareSelf];
-  if ([self isZero] || [b isZero])
-    return [self zero];
-  if ([b isOne])
-    return self;
-  if ([self isOne])
-    return b;
-  if ([b isMinusOne])
-    return [self negateSelf];
-  if ([self isMinusOne])
-    return [b negate];
-
-  if ([self isRecursive])
-    {
-      id res, m = [b asTerm];
-      if (m)
-	{
-	  res = [self multiplySelfTerm:m];
-	  return res;
-	}
-    }
-  else
-    {
-      id res, m = [b asMonomial];
-      if (m)
-	{
-	  res = [self multiplySelfMonomial:m];
-	  return res;
-	}
-    }
-
-  return [self multiply:b];
-}
-
 - square
 {
   if ([self isMinusOne])
@@ -1598,7 +1468,7 @@
 	{
 	  tsqr = [t square];
 	  t2 = [t double];
-	  c = [c addSelf:d multiplyTerm:t2];
+	  c = [c add:d multiplyTerm:t2];
 	  [c insertTerm:tsqr];
 	  [d insertTerm:t];
 	}
@@ -1618,7 +1488,7 @@
 	{
 	  tsqr = [t square];
 	  t2 = [t double];
-	  c = [c addSelf:d multiplyMonomial:t2];
+	  c = [c add:d multiplyMonomial:t2];
 	  [c insertMonomial:tsqr];
 	  [d insertMonomial:t];
 	}
@@ -1729,21 +1599,6 @@
   return [self over:[carrier multiplyScalar:s]];
 }
 
-- multiplySelfScalar:s
-{
-  assert ([scalarZero sameClass:s]);
-
-  if ([s isZero])
-    return [self zero];
-  if ([s isOne])
-    return self;
-  if ([s isMinusOne])
-    return [self negateSelf];
-
-  carrier = [carrier multiplySelfScalar:s];
-  return [self invalidate];
-}
-
 - divideScalar:s
 {
   id tmp;
@@ -1766,29 +1621,6 @@
     }
 }
 
-- divideSelfScalar:s
-{
-  id tmp;
-  assert ([scalarZero sameClass:s]);
-
-  if ([s isZero])
-    return nil;
-  if ([s isOne])
-    return self;
-  if ([s isMinusOne])
-    return [self negateSelf];
-
-  if (tmp = [carrier divideSelfScalar:s])
-    {
-      carrier = tmp;
-      return [self invalidate];
-    }
-  else
-    {
-      return nil;
-    }
-}
-
 - _add:(int)v:B multiplyScalar:b:(int)w
 {
   assert ([self sameClass:B] && [scalarZero sameClass:b] && v * v == 1 && w * w == 1);
@@ -1803,21 +1635,6 @@
   return [self over:[carrier _add:v:[B carrier] multiplyScalar:b:w]];
 }
 
-- _addSelf:(int)v:B multiplyScalar:b:(int)w
-{
-  assert ([self sameClass:B] && [scalarZero sameClass:b] && v * v == 1 && w * w == 1);
-
-  if ([b isZero])
-    return [self _negateSelf:v];
-  if ([b isOne])
-    return [self _addSelf:v:B:+w];
-  if ([b isMinusOne])
-    return [self _addSelf:v:B:-w];
-
-  carrier = [carrier _addSelf:v:[B carrier] multiplyScalar:b:w];
-  return [self invalidate];
-}
-
 - _multiplyScalar:a:(int)v add:B:(int)w
 {
   assert ([scalarZero sameClass:a] && [self sameClass:B] && v * v == 1 && w * w == 1);
@@ -1832,21 +1649,6 @@
   return [self over:[carrier _multiplyScalar:a:v add:[B carrier]:w]];
 }
 
-- _multiplySelfScalar:a:(int)v add:B:(int)w
-{
-  assert ([scalarZero sameClass:a] && [self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([a isZero])
-    return [B _negate:+w];
-  if ([a isOne])
-    return [self _addSelf:+v:B:+w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B:+w];
-
-  carrier = [carrier _multiplySelfScalar:a:v add:[B carrier]:w];
-  return [self invalidate];
-}
-
 - _multiplyScalar:a:(int)v add:B multiplyScalar:b:(int)w
 {
   assert ([scalarZero sameClass:a] && [self sameClass:B] && [scalarZero sameClass:b] && v * v == 1 && w * w == 1);
@@ -1867,26 +1669,6 @@
   return [self over:[carrier _multiplyScalar:a:v add:[B carrier] multiplyScalar:b:w]];
 }
 
-- _multiplySelfScalar:a:(int)v add:B multiplyScalar:b:(int)w
-{
-  assert ([scalarZero sameClass:a] && [self sameClass:B] && [scalarZero sameClass:b] && v * v == 1 && w * w == 1);
-
-  if ([a isZero])
-    return [B _multiplyScalar:b:+w];
-  if ([b isZero])
-    return [self _multiplySelfScalar:a:+v];
-  if ([a isOne])
-    return [self _addSelf:+v:B multiplyScalar:b:+w];
-  if ([b isOne])
-    return [self _multiplySelfScalar:a:+v add:B:+w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B multiplyScalar:b:+w];
-  if ([b isMinusOne])
-    return [self _multiplySelfScalar:a:+v add:B:-w];
-
-  carrier = [carrier _multiplySelfScalar:a:v add:[B carrier] multiplyScalar:b:w];
-  return [self invalidate];
-}
 - multiplyCoefficient:aCoefficient
 {
   if ([self isExpanded])
@@ -1903,20 +1685,6 @@
   return [self over:[carrier multiplyCoefficient:aCoefficient]];
 }
 
-- multiplySelfCoefficient:s
-{
-  assert ([self isRecursive] && [self isVariableDense]);
-
-  if ([s isZero])
-    return [self zero];
-  if ([s isOne])
-    return self;
-  if ([s isMinusOne])
-    return [self negateSelf];
-  carrier = [carrier multiplySelfCoefficient:s];
-  return [self invalidate];
-}
-
 - divideCoefficient:aCoefficient
 {
   id tmp;
@@ -1935,21 +1703,6 @@
   return [self over:tmp];
 }
 
-- divideSelfCoefficient:s
-{
-  id tmp;
-  assert ([self isRecursive] && [self isVariableDense]);
-
-  if ([s isZero])
-    return nil;
-  if ([s isOne])
-    return self;
-  if ([s isMinusOne])
-    return [self negateSelf];
-  tmp = [carrier divideSelfCoefficient:s];
-  return (tmp) ? (carrier = tmp, [self invalidate]) : nil;
-}
-
 - _add:(int)v:B multiplyCoefficient:b:(int)w
 {
   assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
@@ -1964,21 +1717,6 @@
   return [self over:[carrier _add:v:[B carrier] multiplyCoefficient:b:w]];
 }
 
-- _addSelf:(int)v:B multiplyCoefficient:b:(int)w
-{
-  assert ([self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([b isZero])
-    return [self _negateSelf:v];
-  if ([b isOne])
-    return [self _addSelf:v:B:+w];
-  if ([b isMinusOne])
-    return [self _addSelf:v:B:-w];
-
-  carrier = [carrier _addSelf:v:[B carrier] multiplyCoefficient:b:w];
-  return [self invalidate];
-}
-
 - _multiplyCoefficient:a:(int)v add:B:(int)w
 {
   assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
@@ -1993,21 +1731,6 @@
   return [self over:[carrier _multiplyCoefficient:a:v add:[B carrier]:w]];
 }
 
-- _multiplySelfCoefficient:a:(int)v add:B:(int)w
-{
-  assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([a isZero])
-    return [B _negate:+w];
-  if ([a isOne])
-    return [self _addSelf:+v:B:+w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B:+w];
-
-  carrier = [carrier _multiplySelfCoefficient:a:v add:[B carrier]:w];
-  return [self invalidate];
-}
-
 - _multiplyCoefficient:a:(int)v add:B multiplyCoefficient:b:(int)w
 {
   assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
@@ -2028,26 +1751,6 @@
   return [self over:[carrier _multiplyCoefficient:a:v add:[B carrier] multiplyCoefficient:b:w]];
 }
 
-- _multiplySelfCoefficient:a:(int)v add:B multiplyCoefficient:b:(int)w
-{
-  assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([a isZero])
-    return [B _multiplyCoefficient:b:+w];
-  if ([b isZero])
-    return [self _multiplySelfCoefficient:a:+v];
-  if ([a isOne])
-    return [self _addSelf:+v:B multiplyCoefficient:b:+w];
-  if ([b isOne])
-    return [self _multiplySelfCoefficient:a:+v add:B:+w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B multiplyCoefficient:b:+w];
-  if ([b isMinusOne])
-    return [self _multiplySelfCoefficient:a:+v add:B:-w];
-
-  carrier = [carrier _multiplySelfCoefficient:a:v add:[B carrier] multiplyCoefficient:b:w];
-  return [self invalidate];
-}
 - multiplyTerm:aTerm
 {
   if ([self isExpanded])
@@ -2067,23 +1770,6 @@
   return [self over:[carrier multiplyTerm:aTerm]];
 }
 
-- multiplySelfTerm:s
-{
-  if ([self isVariableDense])
-    {
-      if ([s isCoefficient])
-	return [self multiplySelfCoefficient:[s coefficient]];
-    }
-  else
-    {
-      if ([s isCoefficient])
-	return [self multiplySelfScalar:[s coefficient]];
-    }
-
-  carrier = [carrier multiplySelfTerm:s];
-  return [self invalidate];
-}
-
 - divideTerm:aTerm
 {
   id tmp;
@@ -2104,28 +1790,13 @@
   return (tmp = [carrier divideTerm:aTerm]) ? [self over : tmp]:nil;
 }
 
-- divideSelfTerm:s
-{
-  id tmp;
-
-  if ([self isVariableDense])
-    {
-      if ([s isCoefficient])
-	return [self divideSelfCoefficient:[s coefficient]];
-    }
-  else
-    {
-      if ([s isCoefficient])
-	return [self divideSelfScalar:[s coefficient]];
-    }
-
-  return (tmp = [carrier divideSelfTerm:s]) ? (carrier = tmp, [self invalidate]) : nil;
-}
-
 - _add:(int)v:B multiplyTerm:b:(int)w
 {
   assert ([self sameClass:B] && v * v == 1 && w * w == 1);
 
+  if ([B isZero])
+    return [self _negate:v];
+
   if ([self isVariableDense])
     {
       if ([b isCoefficient])
@@ -2140,29 +1811,12 @@
   return [self over:[carrier _add:v:[B carrier] multiplyTerm:b:w]];
 }
 
-- _addSelf:(int)v:B multiplyTerm:b:(int)w
-{
-  assert ([self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([self isVariableDense])
-    {
-      if ([b isCoefficient])
-	return [self _addSelf:v:B multiplyCoefficient:[b coefficient]:w];
-    }
-  else
-    {
-      if ([b isCoefficient])
-	return [self _addSelf:v:B multiplyScalar:[b coefficient]:w];
-    }
-
-  carrier = [carrier _addSelf:v:[B carrier] multiplyTerm:b:w];
-  return [self invalidate];
-}
-
 - _multiplyCoefficient:a:(int)v add:B multiplyTerm:b:(int)w
 {
   assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
 
+  if ([B isZero])
+    return [self _multiplyCoefficient:a:v];
   if ([a isZero])
     return [B _multiplyTerm:b:+w];
   if ([a isOne])
@@ -2175,21 +1829,6 @@
   return [self over:[carrier _multiplyCoefficient:a:v add:[B carrier] multiplyTerm:b:w]];
 }
 
-- _multiplySelfCoefficient:a:(int)v add:B multiplyTerm:b:(int)w
-{
-  assert ([self isVariableDense] && [self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([a isZero])
-    return [B _multiplyTerm:b:+w];
-  if ([a isOne])
-    return [self _addSelf:+v:B multiplyTerm:b:w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B multiplyTerm:b:w];
-  if ([b isCoefficient])
-    return [self _multiplySelfCoefficient:a:v add:B multiplyCoefficient:[b coefficient]:w];
-  carrier = [carrier _multiplySelfCoefficient:a:v add:[B carrier] multiplyTerm:b:w];
-  return [self invalidate];
-}
 - multiplyMonomial:s
 {
   if ([s isScalar])
@@ -2198,15 +1837,6 @@
   return [self over:[carrier multiplyMonomial:s]];
 }
 
-- multiplySelfMonomial:s
-{
-  if ([s isScalar])
-    return [self multiplySelfScalar:[s scalar]];
-
-  carrier = [carrier multiplySelfMonomial:s];
-  return [self invalidate];
-}
-
 - divideMonomial:s
 {
   id tmp;
@@ -2223,23 +1853,6 @@
     }
 }
 
-- divideSelfMonomial:s
-{
-  id tmp;
-  if ([s isScalar])
-    return [self divideSelfScalar:[s scalar]];
-
-  if (tmp = [carrier divideSelfMonomial:s])
-    {
-      carrier = tmp;
-      return [self invalidate];
-    }
-  else
-    {
-      return nil;
-    }
-}
-
 - _add:(int)v:B multiplyMonomial:b:(int)w
 {
   assert ([self sameClass:B] && v * v == 1 && w * w == 1);
@@ -2248,15 +1861,6 @@
   return [self over:[carrier _add:v:[B carrier] multiplyMonomial:b:w]];
 }
 
-- _addSelf:(int)v:B multiplyMonomial:b:(int)w
-{
-  assert ([self sameClass:B] && v * v == 1 && w * w == 1);
-  if ([b isScalar])
-    return [self _addSelf:v:B multiplyScalar:[b scalar]:w];
-  carrier = [carrier _addSelf:v:[B carrier] multiplyMonomial:b:w];
-  return [self invalidate];
-}
-
 - _multiplyScalar:a:(int)v add:B multiplyMonomial:b:(int)w
 {
   assert ([self sameClass:B] && v * v == 1 && w * w == 1);
@@ -2273,22 +1877,6 @@
   return [self over:[carrier _multiplyScalar:a:v add:[B carrier] multiplyMonomial:b:w]];
 }
 
-- _multiplySelfScalar:a:(int)v add:B multiplyMonomial:b:(int)w
-{
-  assert ([self sameClass:B] && v * v == 1 && w * w == 1);
-
-  if ([a isZero])
-    return [B _multiplyMonomial:b:+w];
-  if ([a isOne])
-    return [self _addSelf:+v:B multiplyMonomial:b:w];
-  if ([a isMinusOne])
-    return [self _addSelf:-v:B multiplyMonomial:b:w];
-  if ([b isScalar])
-    return [self _multiplySelfScalar:a:v add:B multiplyScalar:[b coefficient]:w];
-
-  carrier = [carrier _multiplySelfScalar:a:v add:[B carrier] multiplyMonomial:b:w];
-  return [self invalidate];
-}
 
 - remainder:b quotient:(id *)q
 {
@@ -2457,48 +2045,6 @@
     }
 }
 
-- divideSelf:b
-{
-  [self checkSameClass:b];
-
-  if ([b isZero])
-    return nil;
-  if ([self isZero])
-    return [self zero];
-  if (self == b)
-    return [self one];
-  if ([b isOne])
-    return self;
-  if ([b isMinusOne])
-    return [self negateSelf];
-
-  if ([self isEqual:b])
-    return [self one];
-  if ([self isOpposite:b])
-    return [self minusOne];
-
-  if ([self isRecursive])
-    {
-      id res, m = [b asTerm];
-      if (m)
-	{
-	  res = [self divideSelfTerm:m];
-	  return res;
-	}
-    }
-  else
-    {
-      id res, m = [b asMonomial];
-      if (m)
-	{
-	  res = [self divideSelfMonomial:m];
-	  return res;
-	}
-    }
-
-  return [self divide:b];
-}
-
 - remainderReductum:r term:t quotient:(id *)q
 {
   id c, s, inv;
@@ -2534,7 +2080,7 @@
       id ltq = [lt subtractExponents:t];
       if (ltq)
 	{
-	  a = [a subtractSelf:r multiplyTerm:ltq];
+	  a = [a subtract:r multiplyTerm:ltq];
 	  a = [a copy];
 	  if (q)
 	    [*q insertTerm:ltq];
@@ -2561,8 +2107,8 @@
       id ltq = [lt subtractExponents:t];
       if (ltq)
 	{
-	  ltq = [ltq multiplySelfCoefficient:s];
-	  a = [a subtractSelf:r multiplyTerm:ltq];
+	  ltq = [ltq multiplyCoefficient:s];
+	  a = [a subtract:r multiplyTerm:ltq];
 	  a = [a copy];
 	  if (q)
 	    [*q insertTerm:ltq];
@@ -2592,7 +2138,7 @@
 	  id ltqc = [ltq divideCoefficient:s];
 	  if (ltqc)
 	    {
-	      a = [a subtractSelf:r multiplyTerm:ltqc];
+	      a = [a subtract:r multiplyTerm:ltqc];
 	      a = [a copy];
 	      if (q)
 		[*q insertTerm:ltqc];
@@ -2650,7 +2196,7 @@
       id ltq = [lt divideTerms:t];
       if (ltq)
 	{
-	  a = [a subtractSelf:r multiplyMonomial:ltq];
+	  a = [a subtract:r multiplyMonomial:ltq];
 	  a = [a copy];
 	  if (q)
 	    [*q insertMonomial:ltq];
@@ -2677,8 +2223,8 @@
       id ltq = [lt divideTerms:t];
       if (ltq)
 	{
-	  ltq = [ltq multiplySelfScalar:s];
-	  a = [a subtractSelf:r multiplyMonomial:ltq];
+	  ltq = [ltq multiplyScalar:s];
+	  a = [a subtract:r multiplyMonomial:ltq];
 	  a = [a copy];
 	  if (q)
 	    [*q insertMonomial:ltq];
@@ -2708,7 +2254,7 @@
 	  id ltqs = [ltq divideScalar:s];
 	  if (ltqs)
 	    {
-	      a = [a subtractSelf:r multiplyMonomial:ltqs];
+	      a = [a subtract:r multiplyMonomial:ltqs];
 	      a = [a copy];
 	      if (q)
 		[*q insertMonomial:ltqs];
@@ -2808,11 +2354,6 @@
   return [self pseudoRemainder:b quotient:NULL];
 }
 
-- pseudoRemainderSelf:b
-{
-  return [self pseudoRemainder:b];
-}
-
 - pseudoRemainderReductum:r term:t quotient:(id *)q
 {
   id c, s;
@@ -2835,16 +2376,16 @@
     case 1:
       {
 	if (q)
-	  *q = [*q multiplySelfCoefficient:c];
-	c = [c multiplySelfCoefficient:s];
+	  *q = [*q multiplyCoefficient:c];
+	c = [c multiplyCoefficient:s];
 	break;
       }
     default:
       {
 	id e = [s power:d];
 	if (q)
-	  *q = [*q multiplySelfCoefficient:e];
-	c = [c multiplySelfCoefficient:e];
+	  *q = [*q multiplyCoefficient:e];
+	c = [c multiplyCoefficient:e];
 	break;
       }
     }
@@ -2872,11 +2413,11 @@
 	  [a insertTerm:lt];
 	  break;
 	}
-      a = [a multiplySelfCoefficient:s subtract:r multiplyTerm:lt];
+      a = [a multiplyCoefficient:s subtract:r multiplyTerm:lt];
       count++;
       if (q)
 	{
-	  *q = [*q multiplySelfCoefficient:s];
+	  *q = [*q multiplyCoefficient:s];
 	  [*q insertTerm:lt];
 	}
     }
@@ -2903,22 +2444,7 @@
   else
     {
       id content = [self content];
-      self = [self divideScalar:content];
-      return self;
-    }
-}
-
-- divideSelfContent
-{
-  if ([self isZero])
-    {
-      return self;
-    }
-  else
-    {
-      id content = [self content];
-      self = [self divideSelfScalar:content];
-      return self;
+      return [self divideScalar:content];
     }
 }
 
@@ -2951,22 +2477,7 @@
   else
     {
       id content = [self coefficientContent];
-      self = [self divideCoefficient:content];
-      return self;
-    }
-}
-
-- divideSelfCoefficientContent
-{
-  if ([self isZero])
-    {
-      return self;
-    }
-  else
-    {
-      id content = [self coefficientContent];
-      self = [self divideSelfCoefficient:content];
-      return self;
+      return [self divideCoefficient:content];
     }
 }
 
@@ -3031,8 +2542,8 @@
     {
       f = [c multiply:rd];
       if (EVEN (delta))
-	f = [f negateSelf];
-      p = [[p pseudoRemainderSelf:q] divideSelfCoefficient:f];
+	f = [f negate];
+      p = [[p pseudoRemainder:q] divideCoefficient:f];
 
       t = p;
       p = q;
@@ -3041,7 +2552,7 @@
 	break;
       c = [p leadingCoefficient];
       cd = [c power:delta];
-      r = [[r multiplySelf:cd] divideSelf:rd];
+      r = [[r multiply:cd] divide:rd];
       delta = [p degree] - [q degree];
       rd = [r power:delta];
     }
@@ -3071,12 +2582,12 @@
   while ([b notZero])
     {
       id c;
-      a = [a remainderSelf:b];
+      a = [a remainder:b];
       c = a;
       a = b;
       b = c;
     }
-  return [a makeSelfMonic];
+  return [a makeMonic];
 }
 
 - idomGcd:b
@@ -3098,7 +2609,7 @@
   [self doImprovedSubresultant:&a:&b];
 
   g = ([b isZero]) ? [a divideCoefficientContent] : [a one];
-  g = [g multiplySelfCoefficient:gc];
+  g = [g multiplyCoefficient:gc];
 
   return g;
 }
@@ -3200,12 +2711,7 @@
 
   d = [self degree] - [B degree] + 1;
   C = [self pseudoRemainder:B];
-  return (ODD (d) && [B leadingSign] == -1) ? [C negateSelf] : C;
-}
-
-- signedPseudoRemainderSelf:b
-{
-  return [self signedPseudoRemainder:b];
+  return (ODD (d) && [B leadingSign] == -1) ? [C negate] : C;
 }
 
 static int 
@@ -3294,8 +2800,8 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
 
   do
     {
-      f = [[c multiply:rd] absValueSelf];
-      p = [[p signedPseudoRemainderSelf:q] divideSelfCoefficient:f];
+      f = [[c multiply:rd] absValue];
+      p = [[p signedPseudoRemainder:q] divideCoefficient:f];
       sgnlist_add (plus, minus, p, count++);
 
       t = p;
@@ -3305,7 +2811,7 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
 	break;
       c = [p leadingCoefficient];
       cd = [c power:delta];
-      r = [[r multiplySelf:cd] divideSelf:rd];
+      r = [[r multiply:cd] divide:rd];
       delta = [p degree] - [q degree];
       rd = [r power:delta];
     }
@@ -3378,9 +2884,6 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
 - factor
 {
   return [self notImplemented:_cmd];
-  /* don't do it.  you'll just waste time with it.
-   * if you can't resist, keep it private !
-   */
 }
 
 - (BOOL) isSquareFree
@@ -3414,15 +2917,15 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
   for (i = 1; 1; i++)
     {
       dB = [B derive];
-      C = [C subtractSelf:dB];
+      C = [C subtract:dB];
       if ([C isZero])
 	break;
       A = [B gcd:C];
       if ([A notOne])
 	{
-	  B = [B divideSelf:A];
+	  B = [B divide:A];
 	  assert (B);
-	  C = [C divideSelf:A];
+	  C = [C divide:A];
 	  assert (C);
 	  [factors add:[self factorSymbol:A exponent:i]];
 	}
@@ -3448,7 +2951,7 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
       id fj = [[factors at:j] symbol];
       int ej = [[factors at:j] exponent];
       while (--ej)
-	ourGcd = [ourGcd divideSelf:fj];
+	ourGcd = [ourGcd divide:fj];
     }
 
   if ([ourGcd isUnit])
@@ -3459,7 +2962,7 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
     {
       id U, V;
 
-      ourGcd = [ourGcd frobeniusInverseSelf];
+      ourGcd = [ourGcd frobeniusInverse];
       V = [ourGcd factorSquareFree];
       U = factors;
       factors = [CACollection new];
@@ -3485,7 +2988,7 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
 	      gcdjk = [Uj gcd:Vk];
 	      if ([gcdjk notOne])
 		{
-		  Vk = [Vk divideSelf:gcdjk];
+		  Vk = [Vk divide:gcdjk];
 		  [factors add:[self factorSymbol:gcdjk exponent:ej + p * ek]];
 
 		  /* it would be better to construct a new list */
@@ -3579,11 +3082,6 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
   return result;
 }
 
-- truncateSelfAtDegree:(int)d
-{
-  return [self truncateAtDegree:d];
-}
-
 
 - frobenius
 {
@@ -3686,18 +3184,18 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
 	  xn = [x power:f - e];
 	  if (n == 1)
 	    {
-	      px = [px _multiplySelf:xn:+1 add:[term coefficient]:+1];
+	      px = [px _multiply:xn:+1 add:[term coefficient]:+1];
 	    }
 	  else
 	    {
 	      id c = [[term coefficient] evaluateAll:cltnOfScalars];
-	      px = [px _multiplySelf:xn:+1 add:c:+1];
+	      px = [px _multiply:xn:+1 add:c:+1];
 	    }
 	  f = e;
 	}
 
       xn = [x power:f];
-      px = [px multiplySelf:xn];
+      px = [px multiply:xn];
 
       assert ([px sameClass:scalarZero]);
       return px;
@@ -3749,12 +3247,12 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
     {
       e = [term exponent];
       xn = [aPolynomial power:f - e];
-      px = [px _multiplySelf:xn:+1 add:[term coefficient]:+1];
+      px = [px _multiply:xn:+1 add:[term coefficient]:+1];
       f = e;
     }
 
   xn = [aPolynomial power:f];
-  px = [px multiplySelf:xn];
+  px = [px multiply:xn];
 
   return px;
 }
@@ -3781,7 +3279,7 @@ sgnlist_add (int *plus, int *minus, id polynomial, int count)
 	  if (cmp == 0)
 	    {
 	      id power = [aPolynomial power:[term exponent]];
-	      res = [res _addSelf:+1:power multiplyCoefficient:[term coefficient]:+1];
+	      res = [res _add:+1:power multiplyCoefficient:[term coefficient]:+1];
 	    }
 	  else
 	    {
