@@ -1,7 +1,23 @@
 
 /*
- * Computer Algebra Kit (c) 1993,99 by Comp.Alg.Objects.  All Rights Reserved.
- * $Id: term.m,v 1.2 1999/07/06 19:48:12 stes Exp $
+ * Computer Algebra Kit (c) 1993,00 by Comp.Alg.Objects.  All Rights Reserved.
+ * $Id: term.m,v 1.6 2000/10/12 14:40:26 stes Exp $
+ */
+
+/*
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Library General Public License as published 
+ * by the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "cakit.h"
@@ -53,9 +69,7 @@
     [self error:"Scalar must be zero"];
   coefficient = aCoefficient;
   exponent = anExponent;
-  if (exponent == 0 && symbol != nil)
-    [self error:"Symbol must be nil"];
-  symbol = aSymbol;
+  symbol = (exponent == 0) ? nil : aSymbol;
   assert ([self check]);
   return self;
 }
@@ -86,35 +100,46 @@
   return [self scalarZero:aZero coefficient:aCoefficient symbol:aSymbol exponent:anExponent];
 }
 
-- copy
+- copycoefficient
 {
-  self = [super copy];
   if (![coefficient sameClass:scalarZero])
     {
       coefficient = [coefficient copy];
     }
   return self;
 }
-- deepCopy
+
+- copy
+{
+  return [[super copy] copycoefficient];
+}
+- deepcopycoefficient
 {
-  self = [super deepCopy];
   scalarZero = [scalarZero deepCopy];
   coefficient = [coefficient deepCopy];
   if (symbol)
     symbol = [symbol deepCopy];
   return self;
 }
-- clone
+
+- deepCopy
+{
+  return [[super deepCopy] deepcopycoefficient];
+}
+- clonecoefficient
 {
-  self = [super clone];
   coefficient = nil;
   symbol = nil;
   return self;
 }
 
-- coefficient:aCoefficient symbol:aSymbol exponent:(int)anExponent
+- clone
+{
+  return [[super clone] clonecoefficient];
+}
+
+- setcoefficient:aCoefficient symbol:aSymbol exponent:(int)anExponent
 {
-  self = [self clone];
   coefficient = aCoefficient;
   symbol = aSymbol;
   exponent = anExponent;
@@ -122,6 +147,11 @@
   return self;
 }
 
+- coefficient:aCoefficient symbol:aSymbol exponent:(int)anExponent
+{
+  return [[self clone] setcoefficient:aCoefficient symbol:aSymbol exponent:anExponent];
+}
+
 - coefficient:aCoefficient exponent:(int)anExponent
 {
   if (exponent)
@@ -615,7 +645,7 @@
 	    }
 	  else
 	    {
-	      c = [[[b coefficient] subtractScalar:coefficient] negateSelf];
+	      c = [[[b coefficient] subtractScalar:coefficient] negate];
 	    }
 	}
       else
@@ -1136,8 +1166,8 @@
       if (exponent)
 	{
 	  id x = [aPrototype symbol:symbol];
-	  x = [x powerSelf:exponent];
-	  new = [new multiplySelf:x];
+	  x = [x power:exponent];
+	  new = [new multiply:x];
 	}
     }
   else
@@ -1146,8 +1176,8 @@
       if (exponent)
 	{
 	  id x = [aPrototype symbol:symbol];
-	  x = [x powerSelf:exponent];
-	  new = [new multiplySelf:x];
+	  x = [x power:exponent];
+	  new = [new multiply:x];
 	}
     }
 
