@@ -1,7 +1,7 @@
 
 /*
- * Portable Object Compiler (c) 1997,98,99.  All Rights Reserved.
- * $Id: objcrt.m,v 1.15 1999/07/11 16:50:56 stes Exp $
+ * Portable Object Compiler (c) 1997,98,99,00,01.  All Rights Reserved.
+ * $Id: objcrt.m,v 1.4 2001/07/09 19:30:49 stes Exp $
  */
 
 /*
@@ -381,8 +381,7 @@ idassign (id * lhs, id rhs)
   id e = *lhs;
   if (allocFlag)
     {
-      dbg ("%p %s --%i ", e, (e) ? [e name] : "nil", (e) ? _REFCNT (e) : 0);
-      dbg ("%p %s %i++\n", rhs, (rhs) ? [rhs name] : "nil", (rhs) ? _REFCNT (rhs) : 0);
+      dbg ("%p --%i %p %i++\n", e, (e) ? _REFCNT (e) : 0, rhs, (rhs) ? _REFCNT (rhs) : 0);
     }
   if (e)
     {
@@ -405,7 +404,7 @@ idincref (id rhs)
 #ifdef OBJC_REFCNT
   if (allocFlag)
     {
-      dbg ("%p %s %i++\n", rhs, (rhs) ? [rhs name] : "nil", (rhs) ? _REFCNT (rhs) : 0);
+      dbg ("%p %i++\n", rhs, (rhs) ? _REFCNT (rhs) : 0);
     }
   if (rhs)
     {
@@ -421,7 +420,7 @@ iddecref (id e)
 #ifdef OBJC_REFCNT
   if (allocFlag)
     {
-      dbg ("%p %s --%i\n", e, (e) ? [e name] : "nil", (e) ? _REFCNT (e) : 0);
+      dbg ("%p --%i\n", e, (e) ? _REFCNT (e) : 0);
     }
   if (e)
     {
@@ -804,9 +803,13 @@ hashInit ()
 static PHASH 
 hashNew (STR key, PHASH link)
 {
+  int n;
   PHASH obj = (PHASH) OC_Malloc (sizeof (HASH));
+  assert (key != NULL);
   obj->next = link;
-  obj->key = key;
+  n = strlen (key);
+  obj->key = (STR) OC_Malloc (n + 1);
+  strcpy (obj->key, key);
   return obj;
 }
 
@@ -1444,7 +1447,7 @@ newShared (int aSize)
 }
 
 static id 
-newMeta (STR name, id superClass)
+newMeta (STR name, id superClass, int esize)
 {
   id m;
   Cls_t n;
@@ -1456,7 +1459,7 @@ newMeta (STR name, id superClass)
   n->clsSuper = superClass;
   n->clsName = name;		/* strdup ? */
   n->clsTypes = getcls (superClass)->clsTypes;
-  n->clsSizInstance = getcls (superClass)->clsSizInstance;
+  n->clsSizInstance = getcls (superClass)->clsSizInstance + esize;
   n->clsSizDict = 0;
   n->clsDispTable = NULL;
   n->clsStatus = 0;
@@ -1468,12 +1471,12 @@ newMeta (STR name, id superClass)
 }
 
 static id 
-newClass (STR name, id superClass)
+newClass (STR name, id superClass, int eisize, int ecsize)
 {
   id m, meta;
   Cls_t n, c;
 
-  meta = newMeta (name, getcls (superClass)->isa);
+  meta = newMeta (name, getcls (superClass)->isa, ecsize);
   m = newShared (getcls (meta)->clsSizInstance);	/* includes cvars */
   n = getcls (m);
   c = getcls (superClass);
@@ -1482,7 +1485,7 @@ newClass (STR name, id superClass)
   n->clsSuper = superClass;
   n->clsName = name;		/* strdup ? */
   n->clsTypes = c->clsTypes;
-  n->clsSizInstance = c->clsSizInstance;
+  n->clsSizInstance = c->clsSizInstance + eisize;
   n->clsSizDict = 0;
   n->clsDispTable = NULL;
   n->clsStatus = 0;
@@ -1499,8 +1502,8 @@ newModDesc (id * clsLst)
   PMOD mod = (PMOD) OC_Malloc (sizeof (MOD));
 
   mod->modName = "dynamic";
-  mod->modName = "1.7.3";
-  /* not MOD_MORETHANONE (clsLst is not NULL terminated) */
+  mod->modName = "2.3.15";
+  /* not MOD_MORETHANONE (clsLst is not NULL terminated, it's just 1 class) */
   mod->modStatus = 0;
   mod->modMinSel = NULL;
   mod->modMaxSel = NULL;
@@ -1512,7 +1515,7 @@ newModDesc (id * clsLst)
   return mod;
 }
 
-static PMOD dynMod;		/* ugh */
+static PMOD dynMod;		/* ugh - hack to communicate between dynBind and addMod */
 static PMOD 
 dynBIND (void)
 {
@@ -1523,8 +1526,8 @@ static void
 addModEntry (id aCls)
 {
   id *clsLst;
-  Mentry_t entry;
-  entry = (Mentry_t) OC_Malloc (sizeof (struct objcrt_modEntry));
+  Mentry_t entry, sentin;
+  entry = (Mentry_t) OC_Malloc (2 * sizeof (struct objcrt_modEntry));
 
   clsLst = OC_Malloc (sizeof (id));
   *clsLst = aCls;
@@ -1533,15 +1536,68 @@ addModEntry (id aCls)
   entry->modLink = dynBIND;
   entry->modInfo = NULL;
 
+  /* initsels depends on sentinel */
+  sentin = entry + 1;
+  sentin->modLink = NULL;
+  sentin->modInfo = NULL;
+
   loadobjc (entry);		/* dynBIND will use dynMod */
 }
 
 id 
 newsubclass (STR name, id superClass, int ivars, int cvars)
 {
-  id n = newClass (name, superClass);
-  addModEntry (n);
-  return n;
+  return newClass (name, superClass, ivars, cvars);
+}
+
+void 
+linkclass (id aclass)
+{
+  addModEntry (aclass);
+}
+
+void 
+unlinkclass (id aclass)
+{
+  modnode_t m;
+  Mentry_t modPtr;
+
+  if (aclass == nil)
+    return;
+
+  for (m = modnodelist; m; m = m->next)
+    {
+      for (modPtr = m->objcmodules; modPtr && modPtr->modLink; modPtr++)
+	{
+	  id *cls;
+
+	  cls = modPtr->modInfo->modClsLst;
+
+	  if (morethanone (modPtr->modInfo))
+	    {
+	      while (*cls)
+		{
+		  if (*cls == aclass)
+		    {
+		      unloadobjc (modPtr);
+		      return;
+		    }
+		  cls++;
+		}
+	    }
+	  else
+	    {
+	      if (cls)
+		{
+		  if (*cls == aclass)
+		    {
+		      unloadobjc (modPtr);
+		      return;
+		    }
+		}
+	    }
+	}
+    }
 }
 
 /*****************************************************************************
