@@ -1,7 +1,7 @@
 
 /*
  * Portable Object Compiler (c) 1997,98.  All Rights Reserved.
- * $Id: bag.m,v 1.14 1999/09/18 21:22:53 stes Exp $
+ * $Id: bag.m,v 1.3 2001/03/14 20:01:09 stes Exp $
  */
 
 /*
@@ -263,9 +263,13 @@ clear (objbag_t self)
 
 - release
 {
+#ifdef OBJC_REFCNT
   empty ((&value));
   clear ((&value));
   return [super release];
+#else
+  return [self notImplemented:_cmd];
+#endif
 }
 
 /*****************************************************************************
@@ -337,11 +341,15 @@ size (objbag_t self)
     {
       if ([bag contains:anElement])
 	{
+#ifndef OBJC_REFCNT
 	  [aSequence free];
+#endif
 	  return NO;
 	}
     }
+#ifndef OBJC_REFCNT
   [aSequence free];
+#endif
   return YES;
 }
 
@@ -514,7 +522,8 @@ filter (objbag_t self, id obj)
     {
       self->count++;
       *m = 1;
-      return *p = obj;
+      *p = obj;
+      return nil;
     }
 }
 
@@ -525,7 +534,9 @@ filter (objbag_t self, id obj)
       id t = filter ((&value), anObject);
       if (t)
 	{
+#ifndef OBJC_REFCNT
 	  [anObject free];
+#endif
 	  return t;
 	}
       else
@@ -1155,8 +1166,13 @@ delete (objbag_t self, id obj)
 
 - printOn:(IOD)aFile
 {
+#ifndef OBJC_REFCNT
   [[[self eachElement] printOn:aFile] free];
   return self;
+#else
+  [[self eachElement] printOn:aFile];
+  return self;
+#endif
 }
 
 /*****************************************************************************
