@@ -1,7 +1,7 @@
 
 /*
  * Portable Object Compiler (c) 1997,98,99.  All Rights Reserved.
- * $Id: ordcltn.m,v 1.15 1999/10/29 18:33:07 stes Exp $
+ * $Id: ordcltn.m,v 1.2 2000/11/09 22:55:16 stes Exp $
  */
 
 /*
@@ -263,9 +263,13 @@ clear (objcol_t self)
 
 - release
 {
+#ifdef OBJC_REFCNT
   empty ((&value));
   clear ((&value));
   return [super release];
+#else
+  return [self notImplemented:_cmd];
+#endif
 }
 
 /*****************************************************************************
@@ -1331,9 +1335,9 @@ ptrfindstr (id * p, STR s, int n)
   int i;
   for (i = 0; i < n; i++)
     {
-      if ([*p++ isEqualSTR:s])
+      if ([p [i] isEqualSTR:s])
 	{
-	  return *p;
+	  return p [i];
 	}
     }
   return nil;
