@@ -1,7 +1,7 @@
 
 /*
  * Portable Object Compiler (c) 1997,98.  All Rights Reserved.
- * $Id: set.m,v 1.14 1999/09/18 21:22:54 stes Exp $
+ * $Id: set.m,v 1.2 2000/11/09 22:55:16 stes Exp $
  */
 
 /*
@@ -278,9 +278,13 @@ clear (objset_t self)
 
 - release
 {
+#ifdef OBJC_REFCNT
   empty ((&value));
   clear ((&value));
   return [super release];
+#else
+  return [self notImplemented:_cmd];
+#endif
 }
 
 /*****************************************************************************
