@@ -1,7 +1,7 @@
 
 /*
  * Portable Object Compiler (c) 1997,98.  All Rights Reserved.
- * $Id: sortcltn.m,v 1.14 1999/09/18 21:22:54 stes Exp $
+ * $Id: sortcltn.m,v 1.3 2001/03/14 20:01:09 stes Exp $
  */
 
 /*
@@ -252,7 +252,9 @@ destroy (objbbt_t self)
   aSeq = [self eachElement];
   while ((elt = [aSeq next]))
     [aCopy add:[elt deepCopy]];
+#ifndef OBJC_REFCNT
   aSeq = [aSeq free];
+#endif
 
   return aCopy;
 }
@@ -283,8 +285,12 @@ destroy (objbbt_t self)
 
 - release
 {
+#ifdef OBJC_REFCNT
   [self emptyYourself];
   return [super release];
+#else
+  return [self notImplemented:_cmd];
+#endif
 }
 
 /*****************************************************************************
@@ -687,7 +693,11 @@ addnfind (objbbt_t top, id key, SEL selCmp, id cmpBlk)
       if (value.llink)
 	{
 	  id res = addnfind (value.llink, anObject, cmpSel, cmpBlk);
+#ifndef OBJC_REFCNT
 	  return (res == anObject) ? anObject : ([anObject free], res);
+#else
+	  return (res == anObject) ? anObject : res;
+#endif
 	}
       else
 	{
