@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 1998 David Stes.
+ * Copyright (c) 1998,2000 David Stes.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published 
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: deref.m,v 1.2 1999/08/10 22:36:44 stes Exp $
+ * $Id: deref.m,v 1.1.1.1 2000/06/07 21:09:25 stes Exp $
  */
 
 #include "config.h"
@@ -32,6 +32,9 @@
 #include "unyxpr.h"
 #include "deref.h"
 #include "type.h"
+#include "var.h"
+#include "scalar.h"
+#include "aryvar.h"
 
 @implementation Dereference
 
@@ -46,5 +49,39 @@
   return self;
 }
 
+- go
+{
+  id x = [expr go];
+
+  if ([x isKindOf:(id)[ArrayVariable class]]) {
+    return [x at:0];
+  } else {
+    if ([x type] == t_str) {
+      STR s = [x u_str];
+      return [[Scalar new] u_char:*s];
+    }
+    return [self notImplemented];
+  }
+  return self;
+}
+
+- assignvar:v
+{
+  id x = [expr go];
+
+  if ([x isKindOf:(id)[ArrayVariable class]]) {
+    [x at:0 put:v];
+    return v;
+  } else {
+    if ([x type] == t_str) {
+      STR s = [x u_str];
+      *s = [v u_char];
+      return v;
+    }
+    return [self notImplemented];
+  }
+  return self;
+}
+
 @end
  
