@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 1998 David Stes.
+ * Copyright (c) 1998,2000 David Stes.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published 
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dfltstmt.m,v 1.2 1999/08/10 22:36:44 stes Exp $
+ * $Id: dfltstmt.m,v 1.2 2000/10/07 12:13:04 stes Exp $
  */
 
 #include "config.h"
@@ -30,12 +30,17 @@
 #include "node.h"
 #include "stmt.h"
 #include "dfltstmt.h"
+#include "util.h"
+#include "options.h"
 
 @implementation DefaultStmt
 
 - stmt:aStmt
 {
   stmt = aStmt;
+  if (o_refcnt && ![stmt iscompstmt]) {
+    stmt = mkcompstmt(nil, nil, mklist(nil, stmt), nil);
+  }
   return self;
 }
 
@@ -59,5 +64,15 @@
   return self;
 }
 
+- (BOOL)iscaseOf:e
+{
+  return YES; /* anything matches */
+}
+
+- go
+{
+  return [stmt go];
+}
+  
 @end
  
