@@ -16,13 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fundef.h,v 1.2 1999/10/24 21:42:00 stes Exp $
+ * $Id: fundef.h,v 1.1.1.1 2000/06/07 21:09:25 stes Exp $
  */
 
 @interface FunctionDef : Def
 {
+  id unit;
   id datadefspecs, decl, body;
   BOOL isinline, isstatic, isextern, istypedef;
+  id breakpt;
 }
 
 - datadefspecs:s;
@@ -31,6 +33,10 @@
 - gen;
 
 - fcall:x;
+- setbreak:bkpt;
+
+- value;
+- defval:v;
 
 @end
  
