@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: node.h,v 1.2 1999/10/24 21:42:00 stes Exp $
+ * $Id: node.h,v 1.2 2000/08/13 14:00:49 stes Exp $
  */
 
 extern FILE *gfile;
@@ -38,6 +38,7 @@ void gl(int no, char *fn);
 void g_otbvars(void);
 void gstderr(void);
 void gnormal(void);
+char *browsepath(char *filename);
 
 void warn(char *fmt,...);
 void warnat(id sym, char *fmt,...);
@@ -53,6 +54,7 @@ void fatalat(id sym, char *fmt,...);
 - sgen;
 - st80;
 
+- go;
 - (BOOL)isstorageclass;
 
 @end
