@@ -16,39 +16,48 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: scalar.h,v 1.6 1999/10/29 18:33:16 stes Exp $
+ * $Id: scalar.h,v 1.2 2000/09/08 15:02:08 stes Exp $
  */
 
+#define ISSCALARZERO(e) (([e isKindOf:(id)[Scalar class]])?[e isZero]:((e)?0:1))
+
 @interface Scalar : Variable
 {
+   id type;
    union {
-     id u_id;
-     int u_int;
-     long u_long;
-     char u_char;
-     float u_float;
-     double u_double;
-     char *u_charstar;
-     void *u_voidstar;
+     int un_int;
+     unsigned un_uns;
+     long un_long;
+     char un_char;
+     float un_float;
+     double un_double;
+     char *un_str;
+     void *un_voidstar;
+     FILE *un_fp;
    } u;
 }
 
-- u_id:x;
+- type;
+
 - u_int:(int)x;
+- u_uns:(unsigned)x;
 - u_long:(long)x;
 - u_char:(char)x;
 - u_float:(float)x;
 - u_double:(double)x;
-- u_charstar:(char*)x;
+- u_str:(char*)x;
 - u_voidstar:(void*)x;
+- u_fp:(FILE*)x;
 
 - (int)u_int;
 - (long)u_long;
+- (unsigned)u_uns;
 - (char)u_char;
 - (float)u_float;
 - (double)u_double;
-- (char*)u_charstar;
+- (char*)u_str;
 - (void*)u_voidstar;
+- (FILE*)u_fp;
 
 - (BOOL)isZero;
 
