@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: shared.h,v 1.4 1999/10/30 23:45:05 stes Exp $
+ * $Id: shared.h,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 /* methods to which both compound stmt and block expr respond */
@@ -45,3 +45,7 @@
 
 - removeheapvarsfromdatadefs;
 
+- fcall:x;
+- go;
+- switchOn:e;
+
