@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: stkframe.h,v 1.4 1999/11/01 17:58:13 stes Exp $
+ * $Id: stkframe.h,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 extern id topframe;
@@ -24,6 +24,7 @@ extern id topframe;
 @interface Stackframe : Object
 {
   id vals;
+  id def;
   id compound;
   id previous,next;
   BOOL quitframe;
@@ -37,12 +38,15 @@ extern id topframe;
 
 - compound;
 - compound:c;
+- def:m;
 
+- lookupself;
 - lookupval:sym;
 - defval:sym as:scalar;
 
 - lookuplocal:sym;
 
+- exitframe;
 - (BOOL)quitframe;
 - quitframe:(BOOL)x;
 - (BOOL)breakframe;
@@ -54,6 +58,7 @@ extern id topframe;
 - returnval;
 
 - enclosing;
+- printBtOn:(IOD)d;
 
 @end
  
