@@ -16,16 +16,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: var.h,v 1.3 1999/10/30 23:45:05 stes Exp $
+ * $Id: var.h,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 @interface Variable : Node
 {
-  id type;
 }
 
-- type;
-- type:x;
-
 @end
  
