@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: var.m,v 1.2 1999/10/24 12:51:35 stes Exp $
+ * $Id: var.m,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 #include "config.h"
@@ -33,15 +33,5 @@
 
 @implementation Variable
 
-- type
-{
-  return type;
-}
-
-- type:x
-{
-  type = x;return self;
-}
-
 @end
- 
+
