@@ -16,11 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: cmd.h,v 1.2 1999/10/10 12:26:07 stes Exp $
+ * $Id: cmd.h,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 extern id vicicmds;
 
+extern FILE *vistdin;
+extern FILE *vistdout;
+
 @interface Cmd : Object
 {
 }
