@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: cmd.m,v 1.2 1999/10/10 12:26:07 stes Exp $
+ * $Id: cmd.m,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 #include "config.h"
@@ -42,8 +42,22 @@
 
 id vicicmds;
 
+FILE *vistdin;
+FILE *vistdout;
+
 @implementation Cmd
 
++ initialize
+{
+  if (vistdin == NULL) {
+    vistdin = fopen("/dev/tty","r");
+    setbuf(vistdin,NULL);
+    vistdout = fopen("/dev/tty","w");
+    setbuf(vistdout,NULL);
+  }
+  return self;
+}
+
 - go
 {
   return [self subclassResponsibility];
