@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: run.m,v 1.7 1999/10/31 21:50:35 stes Exp $
+ * $Id: run.m,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 #include "config.h"
@@ -43,15 +43,34 @@
 #include <fundef.h>
 #include <options.h>
 
+#include <var.h>
+#include <aryvar.h>
+#include <scalar.h>
+
 #include "cmd.h"
 #include "run.h"
 #include "loaded.h"
+#include "args.h"
 
 @implementation Run
 
 - go
 {
-  [[Loaded resolvestr:o_mainfun] fcall:nil];
+  id c,v;
+  int i,n;
+  n = 1+((viciargs)?[viciargs size]:0);
+  c = [OrdCltn new];
+  [c add:[[Scalar new] u_int:n]];
+  v = [ArrayVariable new:n];
+  [v add:[[Scalar new] u_str:"vici"]];
+  if (viciargs) [v addAll:viciargs];
+  [c add:v];
+  [Loaded reset];
+  [[Loaded resolvestr:o_mainfun] fcall:c];
+  fflush(vistdout);
+  c = [OrdCltn new];
+  [c add:[[Scalar new] u_int:0]];
+  [[Loaded resolvestr:"exit"] fcall:c];
   return self;
 }
 
