@@ -16,15 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: shell.h,v 1.2 1999/10/10 12:00:42 stes Exp $
+ * $Id: shell.h,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 @interface Shell : Cmd
 {
    id what;
+   BOOL edit;
 }
 
+
 - what;
+- edit;
 - what:x;
 
 @end
