@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: shell.m,v 1.4 1999/10/11 20:33:38 stes Exp $
+ * $Id: shell.m,v 1.1.1.1 2000/06/07 21:09:26 stes Exp $
  */
 
 #include "config.h"
@@ -44,6 +44,12 @@
 
 @implementation Shell
 
+- edit
+{
+  edit++;
+  return self;
+}
+
 - what:x
 {
   what = x;
@@ -55,9 +61,23 @@
   return what;
 }
 
+- dosystem:(char*)s
+{
+  system(s);
+  return self;
+}
+
 - go
 {
-  if (what!=nil && [what str]!=NULL) system([what str]);
+  if (what!=nil && [what str]!=NULL) {
+    if (edit) {
+      id buf = [String str:"vi </dev/tty >/dev/tty"];
+      [buf concat:what];
+      [self dosystem:[buf str]];
+    } else {
+      [self dosystem:[what str]];
+    }
+  }
   /* if (automake) */ [Loaded make];
   return self;
 }
