@@ -6,6 +6,8 @@ extern struct modDescriptor *_OBJCBIND_Message();
 extern struct modDescriptor *_OBJCBIND_objcrt();
 extern struct modDescriptor *_OBJCBIND_postlink();
 extern struct modDescriptor *_OBJCBIND_ordcltn();
+extern struct modDescriptor *_OBJCBIND_sortcltn();
+extern struct modDescriptor *_OBJCBIND_treeseq();
 extern struct modDescriptor *_OBJCBIND_Block();
 extern struct modDescriptor *_OBJCBIND_set();
 extern struct modDescriptor *_OBJCBIND_setseq();
@@ -26,6 +28,8 @@ static struct modEntry {
 	{_OBJCBIND_objcrt,0},
 	{_OBJCBIND_postlink,0},
 	{_OBJCBIND_ordcltn,0},
+	{_OBJCBIND_sortcltn,0},
+	{_OBJCBIND_treeseq,0},
 	{_OBJCBIND_Block,0},
 	{_OBJCBIND_set,0},
 	{_OBJCBIND_setseq,0},
